// SPDX-License-Identifier: Unlicense

#include "src/asio.cpp"
#include "src/backup.cpp"
#include "src/base64.cpp"
#include "src/camera.cpp"
#include "src/collection.cpp"
#include "src/coop.cpp"
#include "src/data.cpp"
#include "src/debug.cpp"
#include "src/file.cpp"
#include "src/font.cpp"
#include "src/geometry.cpp"
#include "src/getopt.cpp"
#include "src/global.cpp"
#include "src/glsl.cpp"
#include "src/helper.cpp"
#include "src/hexfin.c"
#include "src/highway.cpp"
#include "src/highway2.cpp"
#include "src/hud.cpp"
#include "src/hud2.cpp"
#include "src/inflate.cpp"
#include "src/ini.cpp"
#include "src/input.cpp"
#include "src/json.cpp"
#include "src/main.cpp"
#include "src/manifest.cpp"
#include "src/model.cpp"
#include "src/midi.cpp"
#include "src/milk.cpp"
#include "src/miniz.cpp"
#include "src/ogg.cpp"
#include "src/opengl.cpp"
#include "src/openxr.cpp"
#include "src/openxr_pico4.cpp"
#include "src/openxr_quest3.cpp"
#include "src/particle.cpp"
#include "src/pcm.cpp"
#include "src/player.cpp"
#include "src/png.cpp"
#include "src/psarc.cpp"
#include "src/recorder.cpp"
#include "src/revorb.cpp"
#include "src/rijndael.cpp"
#include "src/samplerate.c"
#include "src/sampleratesrc_linear.c"
#include "src/sampleratesrc_sinc.c"
#include "src/sampleratesrc_zoh.c"
#include "src/settings.cpp"
#include "src/sfx.cpp"
#include "src/sfxCore.cpp"
#include "src/sfxCore/airWindows/common/airwin_consolidated_base.cpp"
#include "src/sfxCore/airWindows/effects/Acceleration.cpp"
#include "src/sfxCore/airWindows/effects/Acceleration2.cpp"
#include "src/sfxCore/airWindows/effects/Acceleration2Proc.cpp"
#include "src/sfxCore/airWindows/effects/AccelerationProc.cpp"
#include "src/sfxCore/airWindows/effects/ADClip7.cpp"
#include "src/sfxCore/airWindows/effects/ADClip7Proc.cpp"
#include "src/sfxCore/airWindows/effects/ADClip8.cpp"
#include "src/sfxCore/airWindows/effects/ADClip8Proc.cpp"
#include "src/sfxCore/airWindows/effects/ADT.cpp"
#include "src/sfxCore/airWindows/effects/ADTProc.cpp"
#include "src/sfxCore/airWindows/effects/Air.cpp"
#include "src/sfxCore/airWindows/effects/Air2.cpp"
#include "src/sfxCore/airWindows/effects/Air2Proc.cpp"
#include "src/sfxCore/airWindows/effects/Air3.cpp"
#include "src/sfxCore/airWindows/effects/Air3Proc.cpp"
#include "src/sfxCore/airWindows/effects/AirProc.cpp"
#include "src/sfxCore/airWindows/effects/Apicolypse.cpp"
#include "src/sfxCore/airWindows/effects/ApicolypseProc.cpp"
#include "src/sfxCore/airWindows/effects/AQuickVoiceClip.cpp"
#include "src/sfxCore/airWindows/effects/AQuickVoiceClipProc.cpp"
#include "src/sfxCore/airWindows/effects/AtmosphereBuss.cpp"
#include "src/sfxCore/airWindows/effects/AtmosphereBussProc.cpp"
#include "src/sfxCore/airWindows/effects/AtmosphereChannel.cpp"
#include "src/sfxCore/airWindows/effects/AtmosphereChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/Aura.cpp"
#include "src/sfxCore/airWindows/effects/AuraProc.cpp"
#include "src/sfxCore/airWindows/effects/AutoPan.cpp"
#include "src/sfxCore/airWindows/effects/AutoPanProc.cpp"
#include "src/sfxCore/airWindows/effects/Average.cpp"
#include "src/sfxCore/airWindows/effects/AverageProc.cpp"
#include "src/sfxCore/airWindows/effects/AverMatrix.cpp"
#include "src/sfxCore/airWindows/effects/AverMatrixProc.cpp"
#include "src/sfxCore/airWindows/effects/Balanced.cpp"
#include "src/sfxCore/airWindows/effects/BalancedProc.cpp"
#include "src/sfxCore/airWindows/effects/BassAmp.cpp"
#include "src/sfxCore/airWindows/effects/BassAmpProc.cpp"
#include "src/sfxCore/airWindows/effects/BassDrive.cpp"
#include "src/sfxCore/airWindows/effects/BassDriveProc.cpp"
#include "src/sfxCore/airWindows/effects/BassKit.cpp"
#include "src/sfxCore/airWindows/effects/BassKitProc.cpp"
#include "src/sfxCore/airWindows/effects/Baxandall.cpp"
#include "src/sfxCore/airWindows/effects/Baxandall2.cpp"
#include "src/sfxCore/airWindows/effects/Baxandall2Proc.cpp"
#include "src/sfxCore/airWindows/effects/BaxandallProc.cpp"
#include "src/sfxCore/airWindows/effects/Beam.cpp"
#include "src/sfxCore/airWindows/effects/BeamProc.cpp"
#include "src/sfxCore/airWindows/effects/BigAmp.cpp"
#include "src/sfxCore/airWindows/effects/BigAmpProc.cpp"
#include "src/sfxCore/airWindows/effects/Biquad.cpp"
#include "src/sfxCore/airWindows/effects/Biquad2.cpp"
#include "src/sfxCore/airWindows/effects/Biquad2Proc.cpp"
#include "src/sfxCore/airWindows/effects/BiquadDouble.cpp"
#include "src/sfxCore/airWindows/effects/BiquadDoubleProc.cpp"
#include "src/sfxCore/airWindows/effects/BiquadHiLo.cpp"
#include "src/sfxCore/airWindows/effects/BiquadHiLoProc.cpp"
#include "src/sfxCore/airWindows/effects/BiquadNonLin.cpp"
#include "src/sfxCore/airWindows/effects/BiquadNonLinProc.cpp"
#include "src/sfxCore/airWindows/effects/BiquadOneHalf.cpp"
#include "src/sfxCore/airWindows/effects/BiquadOneHalfProc.cpp"
#include "src/sfxCore/airWindows/effects/BiquadPlus.cpp"
#include "src/sfxCore/airWindows/effects/BiquadPlusProc.cpp"
#include "src/sfxCore/airWindows/effects/BiquadProc.cpp"
#include "src/sfxCore/airWindows/effects/BiquadStack.cpp"
#include "src/sfxCore/airWindows/effects/BiquadStackProc.cpp"
#include "src/sfxCore/airWindows/effects/BiquadTriple.cpp"
#include "src/sfxCore/airWindows/effects/BiquadTripleProc.cpp"
#include "src/sfxCore/airWindows/effects/Bite.cpp"
#include "src/sfxCore/airWindows/effects/BiteProc.cpp"
#include "src/sfxCore/airWindows/effects/BitGlitter.cpp"
#include "src/sfxCore/airWindows/effects/BitGlitterProc.cpp"
#include "src/sfxCore/airWindows/effects/BitShiftGain.cpp"
#include "src/sfxCore/airWindows/effects/BitShiftGainProc.cpp"
#include "src/sfxCore/airWindows/effects/BitShiftPan.cpp"
#include "src/sfxCore/airWindows/effects/BitShiftPanProc.cpp"
#include "src/sfxCore/airWindows/effects/BlockParty.cpp"
#include "src/sfxCore/airWindows/effects/BlockPartyProc.cpp"
#include "src/sfxCore/airWindows/effects/BrassRider.cpp"
#include "src/sfxCore/airWindows/effects/BrassRiderProc.cpp"
#include "src/sfxCore/airWindows/effects/BrightAmbience.cpp"
#include "src/sfxCore/airWindows/effects/BrightAmbience2.cpp"
#include "src/sfxCore/airWindows/effects/BrightAmbience2Proc.cpp"
#include "src/sfxCore/airWindows/effects/BrightAmbience3.cpp"
#include "src/sfxCore/airWindows/effects/BrightAmbience3Proc.cpp"
#include "src/sfxCore/airWindows/effects/BrightAmbienceProc.cpp"
#include "src/sfxCore/airWindows/effects/BuildATPDF.cpp"
#include "src/sfxCore/airWindows/effects/BuildATPDFProc.cpp"
#include "src/sfxCore/airWindows/effects/BussColors4.cpp"
#include "src/sfxCore/airWindows/effects/BussColors4Proc.cpp"
#include "src/sfxCore/airWindows/effects/ButterComp.cpp"
#include "src/sfxCore/airWindows/effects/ButterComp2.cpp"
#include "src/sfxCore/airWindows/effects/ButterComp2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ButterCompProc.cpp"
#include "src/sfxCore/airWindows/effects/C5RawBuss.cpp"
#include "src/sfxCore/airWindows/effects/C5RawBussProc.cpp"
#include "src/sfxCore/airWindows/effects/C5RawChannel.cpp"
#include "src/sfxCore/airWindows/effects/C5RawChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/Cabs.cpp"
#include "src/sfxCore/airWindows/effects/CabsProc.cpp"
#include "src/sfxCore/airWindows/effects/Calibre.cpp"
#include "src/sfxCore/airWindows/effects/CalibreProc.cpp"
#include "src/sfxCore/airWindows/effects/Capacitor.cpp"
#include "src/sfxCore/airWindows/effects/Capacitor2.cpp"
#include "src/sfxCore/airWindows/effects/Capacitor2Proc.cpp"
#include "src/sfxCore/airWindows/effects/CapacitorProc.cpp"
#include "src/sfxCore/airWindows/effects/Chamber.cpp"
#include "src/sfxCore/airWindows/effects/Chamber2.cpp"
#include "src/sfxCore/airWindows/effects/Chamber2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ChamberProc.cpp"
#include "src/sfxCore/airWindows/effects/Channel4.cpp"
#include "src/sfxCore/airWindows/effects/Channel4Proc.cpp"
#include "src/sfxCore/airWindows/effects/Channel5.cpp"
#include "src/sfxCore/airWindows/effects/Channel5Proc.cpp"
#include "src/sfxCore/airWindows/effects/Channel6.cpp"
#include "src/sfxCore/airWindows/effects/Channel6Proc.cpp"
#include "src/sfxCore/airWindows/effects/Channel7.cpp"
#include "src/sfxCore/airWindows/effects/Channel7Proc.cpp"
#include "src/sfxCore/airWindows/effects/Channel8.cpp"
#include "src/sfxCore/airWindows/effects/Channel8Proc.cpp"
#include "src/sfxCore/airWindows/effects/Channel9.cpp"
#include "src/sfxCore/airWindows/effects/Channel9Proc.cpp"
#include "src/sfxCore/airWindows/effects/Chorus.cpp"
#include "src/sfxCore/airWindows/effects/ChorusEnsemble.cpp"
#include "src/sfxCore/airWindows/effects/ChorusEnsembleProc.cpp"
#include "src/sfxCore/airWindows/effects/ChorusProc.cpp"
#include "src/sfxCore/airWindows/effects/ChromeOxide.cpp"
#include "src/sfxCore/airWindows/effects/ChromeOxideProc.cpp"
#include "src/sfxCore/airWindows/effects/Cider.cpp"
#include "src/sfxCore/airWindows/effects/CiderProc.cpp"
#include "src/sfxCore/airWindows/effects/ClearCoat.cpp"
#include "src/sfxCore/airWindows/effects/ClearCoatProc.cpp"
#include "src/sfxCore/airWindows/effects/ClipOnly2.cpp"
#include "src/sfxCore/airWindows/effects/ClipOnly2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ClipSoftly.cpp"
#include "src/sfxCore/airWindows/effects/ClipSoftlyProc.cpp"
#include "src/sfxCore/airWindows/effects/CloudCoat.cpp"
#include "src/sfxCore/airWindows/effects/CloudCoatProc.cpp"
#include "src/sfxCore/airWindows/effects/Coils.cpp"
#include "src/sfxCore/airWindows/effects/Coils2.cpp"
#include "src/sfxCore/airWindows/effects/Coils2Proc.cpp"
#include "src/sfxCore/airWindows/effects/CoilsProc.cpp"
#include "src/sfxCore/airWindows/effects/Cojones.cpp"
#include "src/sfxCore/airWindows/effects/CojonesProc.cpp"
#include "src/sfxCore/airWindows/effects/Compresaturator.cpp"
#include "src/sfxCore/airWindows/effects/CompresaturatorProc.cpp"
#include "src/sfxCore/airWindows/effects/Console0Buss.cpp"
#include "src/sfxCore/airWindows/effects/Console0BussProc.cpp"
#include "src/sfxCore/airWindows/effects/Console0Channel.cpp"
#include "src/sfxCore/airWindows/effects/Console0ChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/Console4Buss.cpp"
#include "src/sfxCore/airWindows/effects/Console4BussProc.cpp"
#include "src/sfxCore/airWindows/effects/Console4Channel.cpp"
#include "src/sfxCore/airWindows/effects/Console4ChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/Console5Buss.cpp"
#include "src/sfxCore/airWindows/effects/Console5BussProc.cpp"
#include "src/sfxCore/airWindows/effects/Console5Channel.cpp"
#include "src/sfxCore/airWindows/effects/Console5ChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/Console5DarkCh.cpp"
#include "src/sfxCore/airWindows/effects/Console5DarkChProc.cpp"
#include "src/sfxCore/airWindows/effects/Console6Buss.cpp"
#include "src/sfxCore/airWindows/effects/Console6BussProc.cpp"
#include "src/sfxCore/airWindows/effects/Console6Channel.cpp"
#include "src/sfxCore/airWindows/effects/Console6ChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/Console7Buss.cpp"
#include "src/sfxCore/airWindows/effects/Console7BussProc.cpp"
#include "src/sfxCore/airWindows/effects/Console7Cascade.cpp"
#include "src/sfxCore/airWindows/effects/Console7CascadeProc.cpp"
#include "src/sfxCore/airWindows/effects/Console7Channel.cpp"
#include "src/sfxCore/airWindows/effects/Console7ChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/Console7Crunch.cpp"
#include "src/sfxCore/airWindows/effects/Console7CrunchProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8BussHype.cpp"
#include "src/sfxCore/airWindows/effects/Console8BussHypeProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8BussIn.cpp"
#include "src/sfxCore/airWindows/effects/Console8BussInProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8BussOut.cpp"
#include "src/sfxCore/airWindows/effects/Console8BussOutProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8ChannelHype.cpp"
#include "src/sfxCore/airWindows/effects/Console8ChannelHypeProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8ChannelIn.cpp"
#include "src/sfxCore/airWindows/effects/Console8ChannelInProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8ChannelOut.cpp"
#include "src/sfxCore/airWindows/effects/Console8ChannelOutProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8LiteBuss.cpp"
#include "src/sfxCore/airWindows/effects/Console8LiteBussProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8LiteChannel.cpp"
#include "src/sfxCore/airWindows/effects/Console8LiteChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8SubHype.cpp"
#include "src/sfxCore/airWindows/effects/Console8SubHypeProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8SubIn.cpp"
#include "src/sfxCore/airWindows/effects/Console8SubInProc.cpp"
#include "src/sfxCore/airWindows/effects/Console8SubOut.cpp"
#include "src/sfxCore/airWindows/effects/Console8SubOutProc.cpp"
#include "src/sfxCore/airWindows/effects/Console9Buss.cpp"
#include "src/sfxCore/airWindows/effects/Console9BussProc.cpp"
#include "src/sfxCore/airWindows/effects/Console9Channel.cpp"
#include "src/sfxCore/airWindows/effects/Console9ChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleLABuss.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleLABussProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleLAChannel.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleLAChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleMCBuss.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleMCBussProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleMCChannel.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleMCChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleMDBuss.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleMDBussProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleMDChannel.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleMDChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleXBuss.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleXBussProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleXChannel.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleXChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleXSubIn.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleXSubInProc.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleXSubOut.cpp"
#include "src/sfxCore/airWindows/effects/ConsoleXSubOutProc.cpp"
#include "src/sfxCore/airWindows/effects/ContentHideD.cpp"
#include "src/sfxCore/airWindows/effects/ContentHideDProc.cpp"
#include "src/sfxCore/airWindows/effects/CreamCoat.cpp"
#include "src/sfxCore/airWindows/effects/CreamCoatProc.cpp"
#include "src/sfxCore/airWindows/effects/Creature.cpp"
#include "src/sfxCore/airWindows/effects/CreatureProc.cpp"
#include "src/sfxCore/airWindows/effects/CrickBass.cpp"
#include "src/sfxCore/airWindows/effects/CrickBassProc.cpp"
#include "src/sfxCore/airWindows/effects/CrunchCoat.cpp"
#include "src/sfxCore/airWindows/effects/CrunchCoatProc.cpp"
#include "src/sfxCore/airWindows/effects/CrunchyGrooveWear.cpp"
#include "src/sfxCore/airWindows/effects/CrunchyGrooveWearProc.cpp"
#include "src/sfxCore/airWindows/effects/Crystal.cpp"
#include "src/sfxCore/airWindows/effects/CrystalProc.cpp"
#include "src/sfxCore/airWindows/effects/CStrip.cpp"
#include "src/sfxCore/airWindows/effects/CStrip2.cpp"
#include "src/sfxCore/airWindows/effects/CStrip2Proc.cpp"
#include "src/sfxCore/airWindows/effects/CStripProc.cpp"
#include "src/sfxCore/airWindows/effects/curve.cpp"
#include "src/sfxCore/airWindows/effects/curveProc.cpp"
#include "src/sfxCore/airWindows/effects/Dark.cpp"
#include "src/sfxCore/airWindows/effects/DarkNoise.cpp"
#include "src/sfxCore/airWindows/effects/DarkNoiseProc.cpp"
#include "src/sfxCore/airWindows/effects/DarkProc.cpp"
#include "src/sfxCore/airWindows/effects/DCVoltage.cpp"
#include "src/sfxCore/airWindows/effects/DCVoltageProc.cpp"
#include "src/sfxCore/airWindows/effects/DeBess.cpp"
#include "src/sfxCore/airWindows/effects/DeBessProc.cpp"
#include "src/sfxCore/airWindows/effects/Deckwrecka.cpp"
#include "src/sfxCore/airWindows/effects/DeckwreckaProc.cpp"
#include "src/sfxCore/airWindows/effects/DeEss.cpp"
#include "src/sfxCore/airWindows/effects/DeEssProc.cpp"
#include "src/sfxCore/airWindows/effects/DeHiss.cpp"
#include "src/sfxCore/airWindows/effects/DeHissProc.cpp"
#include "src/sfxCore/airWindows/effects/Density.cpp"
#include "src/sfxCore/airWindows/effects/Density2.cpp"
#include "src/sfxCore/airWindows/effects/Density2Proc.cpp"
#include "src/sfxCore/airWindows/effects/DensityProc.cpp"
#include "src/sfxCore/airWindows/effects/DeRez.cpp"
#include "src/sfxCore/airWindows/effects/DeRez2.cpp"
#include "src/sfxCore/airWindows/effects/DeRez2Proc.cpp"
#include "src/sfxCore/airWindows/effects/DeRez3.cpp"
#include "src/sfxCore/airWindows/effects/DeRez3Proc.cpp"
#include "src/sfxCore/airWindows/effects/DeRezProc.cpp"
#include "src/sfxCore/airWindows/effects/Desk.cpp"
#include "src/sfxCore/airWindows/effects/Desk4.cpp"
#include "src/sfxCore/airWindows/effects/Desk4Proc.cpp"
#include "src/sfxCore/airWindows/effects/DeskProc.cpp"
#include "src/sfxCore/airWindows/effects/DigitalBlack.cpp"
#include "src/sfxCore/airWindows/effects/DigitalBlackProc.cpp"
#include "src/sfxCore/airWindows/effects/Dirt.cpp"
#include "src/sfxCore/airWindows/effects/DirtProc.cpp"
#include "src/sfxCore/airWindows/effects/Discontinuity.cpp"
#include "src/sfxCore/airWindows/effects/DiscontinuityProc.cpp"
#include "src/sfxCore/airWindows/effects/Distance.cpp"
#include "src/sfxCore/airWindows/effects/Distance2.cpp"
#include "src/sfxCore/airWindows/effects/Distance2Proc.cpp"
#include "src/sfxCore/airWindows/effects/Distance3.cpp"
#include "src/sfxCore/airWindows/effects/Distance3Proc.cpp"
#include "src/sfxCore/airWindows/effects/DistanceProc.cpp"
#include "src/sfxCore/airWindows/effects/Distortion.cpp"
#include "src/sfxCore/airWindows/effects/DistortionProc.cpp"
#include "src/sfxCore/airWindows/effects/Ditherbox.cpp"
#include "src/sfxCore/airWindows/effects/DitherboxProc.cpp"
#include "src/sfxCore/airWindows/effects/DitherFloat.cpp"
#include "src/sfxCore/airWindows/effects/DitherFloatProc.cpp"
#include "src/sfxCore/airWindows/effects/DitherMeDiskers.cpp"
#include "src/sfxCore/airWindows/effects/DitherMeDiskersProc.cpp"
#include "src/sfxCore/airWindows/effects/DitherMeTimbers.cpp"
#include "src/sfxCore/airWindows/effects/DitherMeTimbersProc.cpp"
#include "src/sfxCore/airWindows/effects/Doublelay.cpp"
#include "src/sfxCore/airWindows/effects/DoublelayProc.cpp"
#include "src/sfxCore/airWindows/effects/DoublePaul.cpp"
#include "src/sfxCore/airWindows/effects/DoublePaulProc.cpp"
#include "src/sfxCore/airWindows/effects/Drive.cpp"
#include "src/sfxCore/airWindows/effects/DriveProc.cpp"
#include "src/sfxCore/airWindows/effects/DrumSlam.cpp"
#include "src/sfxCore/airWindows/effects/DrumSlamProc.cpp"
#include "src/sfxCore/airWindows/effects/DubCenter.cpp"
#include "src/sfxCore/airWindows/effects/DubCenterProc.cpp"
#include "src/sfxCore/airWindows/effects/Dubly.cpp"
#include "src/sfxCore/airWindows/effects/DublyProc.cpp"
#include "src/sfxCore/airWindows/effects/DubSub.cpp"
#include "src/sfxCore/airWindows/effects/DubSubProc.cpp"
#include "src/sfxCore/airWindows/effects/DustBunny.cpp"
#include "src/sfxCore/airWindows/effects/DustBunnyProc.cpp"
#include "src/sfxCore/airWindows/effects/Dynamics.cpp"
#include "src/sfxCore/airWindows/effects/DynamicsProc.cpp"
#include "src/sfxCore/airWindows/effects/Dyno.cpp"
#include "src/sfxCore/airWindows/effects/DynoProc.cpp"
#include "src/sfxCore/airWindows/effects/Edge.cpp"
#include "src/sfxCore/airWindows/effects/EdgeProc.cpp"
#include "src/sfxCore/airWindows/effects/EdIsDim.cpp"
#include "src/sfxCore/airWindows/effects/EdIsDimProc.cpp"
#include "src/sfxCore/airWindows/effects/Elation.cpp"
#include "src/sfxCore/airWindows/effects/ElationProc.cpp"
#include "src/sfxCore/airWindows/effects/ElectroHat.cpp"
#include "src/sfxCore/airWindows/effects/ElectroHatProc.cpp"
#include "src/sfxCore/airWindows/effects/Energy.cpp"
#include "src/sfxCore/airWindows/effects/Energy2.cpp"
#include "src/sfxCore/airWindows/effects/Energy2Proc.cpp"
#include "src/sfxCore/airWindows/effects/EnergyProc.cpp"
#include "src/sfxCore/airWindows/effects/Ensemble.cpp"
#include "src/sfxCore/airWindows/effects/EnsembleProc.cpp"
#include "src/sfxCore/airWindows/effects/EQ.cpp"
#include "src/sfxCore/airWindows/effects/EQProc.cpp"
#include "src/sfxCore/airWindows/effects/EveryConsole.cpp"
#include "src/sfxCore/airWindows/effects/EveryConsoleProc.cpp"
#include "src/sfxCore/airWindows/effects/EverySlew.cpp"
#include "src/sfxCore/airWindows/effects/EverySlewProc.cpp"
#include "src/sfxCore/airWindows/effects/EveryTrim.cpp"
#include "src/sfxCore/airWindows/effects/EveryTrimProc.cpp"
#include "src/sfxCore/airWindows/effects/Exciter.cpp"
#include "src/sfxCore/airWindows/effects/ExciterProc.cpp"
#include "src/sfxCore/airWindows/effects/Facet.cpp"
#include "src/sfxCore/airWindows/effects/FacetProc.cpp"
#include "src/sfxCore/airWindows/effects/FathomFive.cpp"
#include "src/sfxCore/airWindows/effects/FathomFiveProc.cpp"
#include "src/sfxCore/airWindows/effects/FinalClip.cpp"
#include "src/sfxCore/airWindows/effects/FinalClipProc.cpp"
#include "src/sfxCore/airWindows/effects/FireAmp.cpp"
#include "src/sfxCore/airWindows/effects/FireAmpProc.cpp"
#include "src/sfxCore/airWindows/effects/Flipity.cpp"
#include "src/sfxCore/airWindows/effects/FlipityProc.cpp"
#include "src/sfxCore/airWindows/effects/Floor.cpp"
#include "src/sfxCore/airWindows/effects/FloorProc.cpp"
#include "src/sfxCore/airWindows/effects/Flutter.cpp"
#include "src/sfxCore/airWindows/effects/FlutterProc.cpp"
#include "src/sfxCore/airWindows/effects/Focus.cpp"
#include "src/sfxCore/airWindows/effects/FocusProc.cpp"
#include "src/sfxCore/airWindows/effects/Fracture.cpp"
#include "src/sfxCore/airWindows/effects/Fracture2.cpp"
#include "src/sfxCore/airWindows/effects/Fracture2Proc.cpp"
#include "src/sfxCore/airWindows/effects/FractureProc.cpp"
#include "src/sfxCore/airWindows/effects/FromTape.cpp"
#include "src/sfxCore/airWindows/effects/FromTapeProc.cpp"
#include "src/sfxCore/airWindows/effects/Galactic.cpp"
#include "src/sfxCore/airWindows/effects/Galactic2.cpp"
#include "src/sfxCore/airWindows/effects/Galactic2Proc.cpp"
#include "src/sfxCore/airWindows/effects/GalacticProc.cpp"
#include "src/sfxCore/airWindows/effects/GalacticVibe.cpp"
#include "src/sfxCore/airWindows/effects/GalacticVibeProc.cpp"
#include "src/sfxCore/airWindows/effects/Gatelope.cpp"
#include "src/sfxCore/airWindows/effects/GatelopeProc.cpp"
#include "src/sfxCore/airWindows/effects/GlitchShifter.cpp"
#include "src/sfxCore/airWindows/effects/GlitchShifterProc.cpp"
#include "src/sfxCore/airWindows/effects/GoldenSlew.cpp"
#include "src/sfxCore/airWindows/effects/GoldenSlewProc.cpp"
#include "src/sfxCore/airWindows/effects/Golem.cpp"
#include "src/sfxCore/airWindows/effects/GolemProc.cpp"
#include "src/sfxCore/airWindows/effects/GrindAmp.cpp"
#include "src/sfxCore/airWindows/effects/GrindAmpProc.cpp"
#include "src/sfxCore/airWindows/effects/Gringer.cpp"
#include "src/sfxCore/airWindows/effects/GringerProc.cpp"
#include "src/sfxCore/airWindows/effects/GrooveWear.cpp"
#include "src/sfxCore/airWindows/effects/GrooveWearProc.cpp"
#include "src/sfxCore/airWindows/effects/GuitarConditioner.cpp"
#include "src/sfxCore/airWindows/effects/GuitarConditionerProc.cpp"
#include "src/sfxCore/airWindows/effects/HardVacuum.cpp"
#include "src/sfxCore/airWindows/effects/HardVacuumProc.cpp"
#include "src/sfxCore/airWindows/effects/Hermepass.cpp"
#include "src/sfxCore/airWindows/effects/HermepassProc.cpp"
#include "src/sfxCore/airWindows/effects/HermeTrim.cpp"
#include "src/sfxCore/airWindows/effects/HermeTrimProc.cpp"
#include "src/sfxCore/airWindows/effects/HighGlossDither.cpp"
#include "src/sfxCore/airWindows/effects/HighGlossDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/HighImpact.cpp"
#include "src/sfxCore/airWindows/effects/HighImpactProc.cpp"
#include "src/sfxCore/airWindows/effects/Highpass.cpp"
#include "src/sfxCore/airWindows/effects/Highpass2.cpp"
#include "src/sfxCore/airWindows/effects/Highpass2Proc.cpp"
#include "src/sfxCore/airWindows/effects/HighpassProc.cpp"
#include "src/sfxCore/airWindows/effects/Holt.cpp"
#include "src/sfxCore/airWindows/effects/Holt2.cpp"
#include "src/sfxCore/airWindows/effects/Holt2Proc.cpp"
#include "src/sfxCore/airWindows/effects/HoltProc.cpp"
#include "src/sfxCore/airWindows/effects/Hombre.cpp"
#include "src/sfxCore/airWindows/effects/HombreProc.cpp"
#include "src/sfxCore/airWindows/effects/Huge.cpp"
#include "src/sfxCore/airWindows/effects/HugeProc.cpp"
#include "src/sfxCore/airWindows/effects/Hull.cpp"
#include "src/sfxCore/airWindows/effects/Hull2.cpp"
#include "src/sfxCore/airWindows/effects/Hull2Proc.cpp"
#include "src/sfxCore/airWindows/effects/HullProc.cpp"
#include "src/sfxCore/airWindows/effects/Hype.cpp"
#include "src/sfxCore/airWindows/effects/HypeProc.cpp"
#include "src/sfxCore/airWindows/effects/Hypersonic.cpp"
#include "src/sfxCore/airWindows/effects/HypersonicProc.cpp"
#include "src/sfxCore/airWindows/effects/HypersonX.cpp"
#include "src/sfxCore/airWindows/effects/HypersonXProc.cpp"
#include "src/sfxCore/airWindows/effects/Infinity.cpp"
#include "src/sfxCore/airWindows/effects/Infinity2.cpp"
#include "src/sfxCore/airWindows/effects/Infinity2Proc.cpp"
#include "src/sfxCore/airWindows/effects/InfinityProc.cpp"
#include "src/sfxCore/airWindows/effects/Inflamer.cpp"
#include "src/sfxCore/airWindows/effects/InflamerProc.cpp"
#include "src/sfxCore/airWindows/effects/Infrasonic.cpp"
#include "src/sfxCore/airWindows/effects/InfrasonicProc.cpp"
#include "src/sfxCore/airWindows/effects/Interstage.cpp"
#include "src/sfxCore/airWindows/effects/InterstageProc.cpp"
#include "src/sfxCore/airWindows/effects/IronOxide5.cpp"
#include "src/sfxCore/airWindows/effects/IronOxide5Proc.cpp"
#include "src/sfxCore/airWindows/effects/IronOxideClassic.cpp"
#include "src/sfxCore/airWindows/effects/IronOxideClassic2.cpp"
#include "src/sfxCore/airWindows/effects/IronOxideClassic2Proc.cpp"
#include "src/sfxCore/airWindows/effects/IronOxideClassicProc.cpp"
#include "src/sfxCore/airWindows/effects/Isolator.cpp"
#include "src/sfxCore/airWindows/effects/Isolator2.cpp"
#include "src/sfxCore/airWindows/effects/Isolator2Proc.cpp"
#include "src/sfxCore/airWindows/effects/IsolatorProc.cpp"
#include "src/sfxCore/airWindows/effects/Kalman.cpp"
#include "src/sfxCore/airWindows/effects/KalmanProc.cpp"
#include "src/sfxCore/airWindows/effects/kCathedral.cpp"
#include "src/sfxCore/airWindows/effects/kCathedral2.cpp"
#include "src/sfxCore/airWindows/effects/kCathedral2Proc.cpp"
#include "src/sfxCore/airWindows/effects/kCathedral3.cpp"
#include "src/sfxCore/airWindows/effects/kCathedral3Proc.cpp"
#include "src/sfxCore/airWindows/effects/kCathedralProc.cpp"
#include "src/sfxCore/airWindows/effects/kChamberAR.cpp"
#include "src/sfxCore/airWindows/effects/kChamberARProc.cpp"
#include "src/sfxCore/airWindows/effects/kGuitarHall.cpp"
#include "src/sfxCore/airWindows/effects/kGuitarHallProc.cpp"
#include "src/sfxCore/airWindows/effects/kPlate140.cpp"
#include "src/sfxCore/airWindows/effects/kPlate140Proc.cpp"
#include "src/sfxCore/airWindows/effects/kPlate240.cpp"
#include "src/sfxCore/airWindows/effects/kPlate240Proc.cpp"
#include "src/sfxCore/airWindows/effects/kPlateA.cpp"
#include "src/sfxCore/airWindows/effects/kPlateAProc.cpp"
#include "src/sfxCore/airWindows/effects/kPlateB.cpp"
#include "src/sfxCore/airWindows/effects/kPlateBProc.cpp"
#include "src/sfxCore/airWindows/effects/kPlateC.cpp"
#include "src/sfxCore/airWindows/effects/kPlateCProc.cpp"
#include "src/sfxCore/airWindows/effects/kPlateD.cpp"
#include "src/sfxCore/airWindows/effects/kPlateDProc.cpp"
#include "src/sfxCore/airWindows/effects/LeadAmp.cpp"
#include "src/sfxCore/airWindows/effects/LeadAmpProc.cpp"
#include "src/sfxCore/airWindows/effects/LeftoMono.cpp"
#include "src/sfxCore/airWindows/effects/LeftoMonoProc.cpp"
#include "src/sfxCore/airWindows/effects/LilAmp.cpp"
#include "src/sfxCore/airWindows/effects/LilAmpProc.cpp"
#include "src/sfxCore/airWindows/effects/Logical4.cpp"
#include "src/sfxCore/airWindows/effects/Logical4Proc.cpp"
#include "src/sfxCore/airWindows/effects/Loud.cpp"
#include "src/sfxCore/airWindows/effects/LoudProc.cpp"
#include "src/sfxCore/airWindows/effects/Lowpass.cpp"
#include "src/sfxCore/airWindows/effects/Lowpass2.cpp"
#include "src/sfxCore/airWindows/effects/Lowpass2Proc.cpp"
#include "src/sfxCore/airWindows/effects/LowpassProc.cpp"
#include "src/sfxCore/airWindows/effects/LRFlipTimer.cpp"
#include "src/sfxCore/airWindows/effects/LRFlipTimerProc.cpp"
#include "src/sfxCore/airWindows/effects/Luxor.cpp"
#include "src/sfxCore/airWindows/effects/LuxorProc.cpp"
#include "src/sfxCore/airWindows/effects/MackEQ.cpp"
#include "src/sfxCore/airWindows/effects/MackEQProc.cpp"
#include "src/sfxCore/airWindows/effects/Mackity.cpp"
#include "src/sfxCore/airWindows/effects/MackityProc.cpp"
#include "src/sfxCore/airWindows/effects/MatrixVerb.cpp"
#include "src/sfxCore/airWindows/effects/MatrixVerbProc.cpp"
#include "src/sfxCore/airWindows/effects/Melt.cpp"
#include "src/sfxCore/airWindows/effects/MeltProc.cpp"
#include "src/sfxCore/airWindows/effects/MidAmp.cpp"
#include "src/sfxCore/airWindows/effects/MidAmpProc.cpp"
#include "src/sfxCore/airWindows/effects/MidSide.cpp"
#include "src/sfxCore/airWindows/effects/MidSideProc.cpp"
#include "src/sfxCore/airWindows/effects/Mojo.cpp"
#include "src/sfxCore/airWindows/effects/MojoProc.cpp"
#include "src/sfxCore/airWindows/effects/Monitoring.cpp"
#include "src/sfxCore/airWindows/effects/Monitoring2.cpp"
#include "src/sfxCore/airWindows/effects/Monitoring2Proc.cpp"
#include "src/sfxCore/airWindows/effects/Monitoring3.cpp"
#include "src/sfxCore/airWindows/effects/Monitoring3Proc.cpp"
#include "src/sfxCore/airWindows/effects/MonitoringProc.cpp"
#include "src/sfxCore/airWindows/effects/MoNoam.cpp"
#include "src/sfxCore/airWindows/effects/MoNoamProc.cpp"
#include "src/sfxCore/airWindows/effects/MSFlipTimer.cpp"
#include "src/sfxCore/airWindows/effects/MSFlipTimerProc.cpp"
#include "src/sfxCore/airWindows/effects/MultiBandDistortion.cpp"
#include "src/sfxCore/airWindows/effects/MultiBandDistortionProc.cpp"
#include "src/sfxCore/airWindows/effects/MV.cpp"
#include "src/sfxCore/airWindows/effects/MV2.cpp"
#include "src/sfxCore/airWindows/effects/MV2Proc.cpp"
#include "src/sfxCore/airWindows/effects/MVProc.cpp"
#include "src/sfxCore/airWindows/effects/NaturalizeDither.cpp"
#include "src/sfxCore/airWindows/effects/NaturalizeDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/NCSeventeen.cpp"
#include "src/sfxCore/airWindows/effects/NCSeventeenProc.cpp"
#include "src/sfxCore/airWindows/effects/Neverland.cpp"
#include "src/sfxCore/airWindows/effects/NeverlandProc.cpp"
#include "src/sfxCore/airWindows/effects/Nikola.cpp"
#include "src/sfxCore/airWindows/effects/NikolaProc.cpp"
#include "src/sfxCore/airWindows/effects/NodeDither.cpp"
#include "src/sfxCore/airWindows/effects/NodeDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/Noise.cpp"
#include "src/sfxCore/airWindows/effects/NoiseProc.cpp"
#include "src/sfxCore/airWindows/effects/NonlinearSpace.cpp"
#include "src/sfxCore/airWindows/effects/NonlinearSpaceProc.cpp"
#include "src/sfxCore/airWindows/effects/NotJustAnotherCD.cpp"
#include "src/sfxCore/airWindows/effects/NotJustAnotherCDProc.cpp"
#include "src/sfxCore/airWindows/effects/NotJustAnotherDither.cpp"
#include "src/sfxCore/airWindows/effects/NotJustAnotherDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/OneCornerClip.cpp"
#include "src/sfxCore/airWindows/effects/OneCornerClipProc.cpp"
#include "src/sfxCore/airWindows/effects/OrbitKick.cpp"
#include "src/sfxCore/airWindows/effects/OrbitKickProc.cpp"
#include "src/sfxCore/airWindows/effects/Overheads.cpp"
#include "src/sfxCore/airWindows/effects/OverheadsProc.cpp"
#include "src/sfxCore/airWindows/effects/Pafnuty.cpp"
#include "src/sfxCore/airWindows/effects/Pafnuty2.cpp"
#include "src/sfxCore/airWindows/effects/Pafnuty2Proc.cpp"
#include "src/sfxCore/airWindows/effects/PafnutyProc.cpp"
#include "src/sfxCore/airWindows/effects/Parametric.cpp"
#include "src/sfxCore/airWindows/effects/ParametricProc.cpp"
#include "src/sfxCore/airWindows/effects/PaulDither.cpp"
#include "src/sfxCore/airWindows/effects/PaulDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/PaulWide.cpp"
#include "src/sfxCore/airWindows/effects/PaulWideProc.cpp"
#include "src/sfxCore/airWindows/effects/PDBuss.cpp"
#include "src/sfxCore/airWindows/effects/PDBussProc.cpp"
#include "src/sfxCore/airWindows/effects/PDChannel.cpp"
#include "src/sfxCore/airWindows/effects/PDChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/PeaksOnly.cpp"
#include "src/sfxCore/airWindows/effects/PeaksOnlyProc.cpp"
#include "src/sfxCore/airWindows/effects/Pear.cpp"
#include "src/sfxCore/airWindows/effects/Pear2.cpp"
#include "src/sfxCore/airWindows/effects/Pear2Proc.cpp"
#include "src/sfxCore/airWindows/effects/PearProc.cpp"
#include "src/sfxCore/airWindows/effects/PhaseNudge.cpp"
#include "src/sfxCore/airWindows/effects/PhaseNudgeProc.cpp"
#include "src/sfxCore/airWindows/effects/PitchDelay.cpp"
#include "src/sfxCore/airWindows/effects/PitchDelayProc.cpp"
#include "src/sfxCore/airWindows/effects/PitchNasty.cpp"
#include "src/sfxCore/airWindows/effects/PitchNastyProc.cpp"
#include "src/sfxCore/airWindows/effects/PlatinumSlew.cpp"
#include "src/sfxCore/airWindows/effects/PlatinumSlewProc.cpp"
#include "src/sfxCore/airWindows/effects/PocketVerbs.cpp"
#include "src/sfxCore/airWindows/effects/PocketVerbsProc.cpp"
#include "src/sfxCore/airWindows/effects/Pockey.cpp"
#include "src/sfxCore/airWindows/effects/Pockey2.cpp"
#include "src/sfxCore/airWindows/effects/Pockey2Proc.cpp"
#include "src/sfxCore/airWindows/effects/PockeyProc.cpp"
#include "src/sfxCore/airWindows/effects/Podcast.cpp"
#include "src/sfxCore/airWindows/effects/PodcastDeluxe.cpp"
#include "src/sfxCore/airWindows/effects/PodcastDeluxeProc.cpp"
#include "src/sfxCore/airWindows/effects/PodcastProc.cpp"
#include "src/sfxCore/airWindows/effects/Point.cpp"
#include "src/sfxCore/airWindows/effects/PointProc.cpp"
#include "src/sfxCore/airWindows/effects/Pop.cpp"
#include "src/sfxCore/airWindows/effects/Pop2.cpp"
#include "src/sfxCore/airWindows/effects/Pop2Proc.cpp"
#include "src/sfxCore/airWindows/effects/Pop3.cpp"
#include "src/sfxCore/airWindows/effects/Pop3Proc.cpp"
#include "src/sfxCore/airWindows/effects/PopProc.cpp"
#include "src/sfxCore/airWindows/effects/PowerSag.cpp"
#include "src/sfxCore/airWindows/effects/PowerSag2.cpp"
#include "src/sfxCore/airWindows/effects/PowerSag2Proc.cpp"
#include "src/sfxCore/airWindows/effects/PowerSagProc.cpp"
#include "src/sfxCore/airWindows/effects/Precious.cpp"
#include "src/sfxCore/airWindows/effects/PreciousProc.cpp"
#include "src/sfxCore/airWindows/effects/Preponderant.cpp"
#include "src/sfxCore/airWindows/effects/PreponderantProc.cpp"
#include "src/sfxCore/airWindows/effects/Pressure4.cpp"
#include "src/sfxCore/airWindows/effects/Pressure4Proc.cpp"
#include "src/sfxCore/airWindows/effects/Pressure5.cpp"
#include "src/sfxCore/airWindows/effects/Pressure5Proc.cpp"
#include "src/sfxCore/airWindows/effects/PurestAir.cpp"
#include "src/sfxCore/airWindows/effects/PurestAirProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsole2Buss.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsole2BussProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsole2Channel.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsole2ChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsole3Buss.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsole3BussProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsole3Channel.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsole3ChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsoleBuss.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsoleBussProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsoleChannel.cpp"
#include "src/sfxCore/airWindows/effects/PurestConsoleChannelProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestDrive.cpp"
#include "src/sfxCore/airWindows/effects/PurestDriveProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestEcho.cpp"
#include "src/sfxCore/airWindows/effects/PurestEchoProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestFade.cpp"
#include "src/sfxCore/airWindows/effects/PurestFadeProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestGain.cpp"
#include "src/sfxCore/airWindows/effects/PurestGainProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestSquish.cpp"
#include "src/sfxCore/airWindows/effects/PurestSquishProc.cpp"
#include "src/sfxCore/airWindows/effects/PurestWarm.cpp"
#include "src/sfxCore/airWindows/effects/PurestWarm2.cpp"
#include "src/sfxCore/airWindows/effects/PurestWarm2Proc.cpp"
#include "src/sfxCore/airWindows/effects/PurestWarmProc.cpp"
#include "src/sfxCore/airWindows/effects/Pyewacket.cpp"
#include "src/sfxCore/airWindows/effects/PyewacketProc.cpp"
#include "src/sfxCore/airWindows/effects/RawGlitters.cpp"
#include "src/sfxCore/airWindows/effects/RawGlittersProc.cpp"
#include "src/sfxCore/airWindows/effects/RawTimbers.cpp"
#include "src/sfxCore/airWindows/effects/RawTimbersProc.cpp"
#include "src/sfxCore/airWindows/effects/Recurve.cpp"
#include "src/sfxCore/airWindows/effects/RecurveProc.cpp"
#include "src/sfxCore/airWindows/effects/Remap.cpp"
#include "src/sfxCore/airWindows/effects/RemapProc.cpp"
#include "src/sfxCore/airWindows/effects/ResEQ.cpp"
#include "src/sfxCore/airWindows/effects/ResEQ2.cpp"
#include "src/sfxCore/airWindows/effects/ResEQ2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ResEQProc.cpp"
#include "src/sfxCore/airWindows/effects/Reverb.cpp"
#include "src/sfxCore/airWindows/effects/ReverbProc.cpp"
#include "src/sfxCore/airWindows/effects/Righteous4.cpp"
#include "src/sfxCore/airWindows/effects/Righteous4Proc.cpp"
#include "src/sfxCore/airWindows/effects/RightoMono.cpp"
#include "src/sfxCore/airWindows/effects/RightoMonoProc.cpp"
#include "src/sfxCore/airWindows/effects/SampleDelay.cpp"
#include "src/sfxCore/airWindows/effects/SampleDelayProc.cpp"
#include "src/sfxCore/airWindows/effects/Shape.cpp"
#include "src/sfxCore/airWindows/effects/ShapeProc.cpp"
#include "src/sfxCore/airWindows/effects/ShortBuss.cpp"
#include "src/sfxCore/airWindows/effects/ShortBussProc.cpp"
#include "src/sfxCore/airWindows/effects/SideDull.cpp"
#include "src/sfxCore/airWindows/effects/SideDullProc.cpp"
#include "src/sfxCore/airWindows/effects/Sidepass.cpp"
#include "src/sfxCore/airWindows/effects/SidepassProc.cpp"
#include "src/sfxCore/airWindows/effects/Silhouette.cpp"
#include "src/sfxCore/airWindows/effects/SilhouetteProc.cpp"
#include "src/sfxCore/airWindows/effects/Sinew.cpp"
#include "src/sfxCore/airWindows/effects/SinewProc.cpp"
#include "src/sfxCore/airWindows/effects/SingleEndedTriode.cpp"
#include "src/sfxCore/airWindows/effects/SingleEndedTriodeProc.cpp"
#include "src/sfxCore/airWindows/effects/Slew.cpp"
#include "src/sfxCore/airWindows/effects/Slew2.cpp"
#include "src/sfxCore/airWindows/effects/Slew2Proc.cpp"
#include "src/sfxCore/airWindows/effects/Slew3.cpp"
#include "src/sfxCore/airWindows/effects/Slew3Proc.cpp"
#include "src/sfxCore/airWindows/effects/SlewOnly.cpp"
#include "src/sfxCore/airWindows/effects/SlewOnlyProc.cpp"
#include "src/sfxCore/airWindows/effects/SlewProc.cpp"
#include "src/sfxCore/airWindows/effects/SlewSonic.cpp"
#include "src/sfxCore/airWindows/effects/SlewSonicProc.cpp"
#include "src/sfxCore/airWindows/effects/Smooth.cpp"
#include "src/sfxCore/airWindows/effects/SmoothProc.cpp"
#include "src/sfxCore/airWindows/effects/SoftGate.cpp"
#include "src/sfxCore/airWindows/effects/SoftGateProc.cpp"
#include "src/sfxCore/airWindows/effects/SpatializeDither.cpp"
#include "src/sfxCore/airWindows/effects/SpatializeDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/Spiral.cpp"
#include "src/sfxCore/airWindows/effects/Spiral2.cpp"
#include "src/sfxCore/airWindows/effects/Spiral2Proc.cpp"
#include "src/sfxCore/airWindows/effects/SpiralProc.cpp"
#include "src/sfxCore/airWindows/effects/Srsly.cpp"
#include "src/sfxCore/airWindows/effects/Srsly2.cpp"
#include "src/sfxCore/airWindows/effects/Srsly2Proc.cpp"
#include "src/sfxCore/airWindows/effects/Srsly3.cpp"
#include "src/sfxCore/airWindows/effects/Srsly3Proc.cpp"
#include "src/sfxCore/airWindows/effects/SrslyProc.cpp"
#include "src/sfxCore/airWindows/effects/StarChild.cpp"
#include "src/sfxCore/airWindows/effects/StarChild2.cpp"
#include "src/sfxCore/airWindows/effects/StarChild2Proc.cpp"
#include "src/sfxCore/airWindows/effects/StarChildProc.cpp"
#include "src/sfxCore/airWindows/effects/StereoChorus.cpp"
#include "src/sfxCore/airWindows/effects/StereoChorusProc.cpp"
#include "src/sfxCore/airWindows/effects/StereoDoubler.cpp"
#include "src/sfxCore/airWindows/effects/StereoDoublerProc.cpp"
#include "src/sfxCore/airWindows/effects/StereoEnsemble.cpp"
#include "src/sfxCore/airWindows/effects/StereoEnsembleProc.cpp"
#include "src/sfxCore/airWindows/effects/StereoFX.cpp"
#include "src/sfxCore/airWindows/effects/StereoFXProc.cpp"
#include "src/sfxCore/airWindows/effects/Stonefire.cpp"
#include "src/sfxCore/airWindows/effects/StoneFireComp.cpp"
#include "src/sfxCore/airWindows/effects/StoneFireCompProc.cpp"
#include "src/sfxCore/airWindows/effects/StonefireProc.cpp"
#include "src/sfxCore/airWindows/effects/StudioTan.cpp"
#include "src/sfxCore/airWindows/effects/StudioTanProc.cpp"
#include "src/sfxCore/airWindows/effects/SubsOnly.cpp"
#include "src/sfxCore/airWindows/effects/SubsOnlyProc.cpp"
#include "src/sfxCore/airWindows/effects/SubTight.cpp"
#include "src/sfxCore/airWindows/effects/SubTightProc.cpp"
#include "src/sfxCore/airWindows/effects/Surge.cpp"
#include "src/sfxCore/airWindows/effects/SurgeProc.cpp"
#include "src/sfxCore/airWindows/effects/SurgeTide.cpp"
#include "src/sfxCore/airWindows/effects/SurgeTideProc.cpp"
#include "src/sfxCore/airWindows/effects/Sweeten.cpp"
#include "src/sfxCore/airWindows/effects/SweetenProc.cpp"
#include "src/sfxCore/airWindows/effects/Swell.cpp"
#include "src/sfxCore/airWindows/effects/SwellProc.cpp"
#include "src/sfxCore/airWindows/effects/Tape.cpp"
#include "src/sfxCore/airWindows/effects/TapeDelay.cpp"
#include "src/sfxCore/airWindows/effects/TapeDelay2.cpp"
#include "src/sfxCore/airWindows/effects/TapeDelay2Proc.cpp"
#include "src/sfxCore/airWindows/effects/TapeDelayProc.cpp"
#include "src/sfxCore/airWindows/effects/TapeDither.cpp"
#include "src/sfxCore/airWindows/effects/TapeDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/TapeDust.cpp"
#include "src/sfxCore/airWindows/effects/TapeDustProc.cpp"
#include "src/sfxCore/airWindows/effects/TapeFat.cpp"
#include "src/sfxCore/airWindows/effects/TapeFatProc.cpp"
#include "src/sfxCore/airWindows/effects/TapeProc.cpp"
#include "src/sfxCore/airWindows/effects/Texturize.cpp"
#include "src/sfxCore/airWindows/effects/TexturizeMS.cpp"
#include "src/sfxCore/airWindows/effects/TexturizeMSProc.cpp"
#include "src/sfxCore/airWindows/effects/TexturizeProc.cpp"
#include "src/sfxCore/airWindows/effects/Thunder.cpp"
#include "src/sfxCore/airWindows/effects/ThunderProc.cpp"
#include "src/sfxCore/airWindows/effects/ToneSlant.cpp"
#include "src/sfxCore/airWindows/effects/ToneSlantProc.cpp"
#include "src/sfxCore/airWindows/effects/ToTape5.cpp"
#include "src/sfxCore/airWindows/effects/ToTape5Proc.cpp"
#include "src/sfxCore/airWindows/effects/ToTape6.cpp"
#include "src/sfxCore/airWindows/effects/ToTape6Proc.cpp"
#include "src/sfxCore/airWindows/effects/ToVinyl4.cpp"
#include "src/sfxCore/airWindows/effects/ToVinyl4Proc.cpp"
#include "src/sfxCore/airWindows/effects/TPDFDither.cpp"
#include "src/sfxCore/airWindows/effects/TPDFDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/TPDFWide.cpp"
#include "src/sfxCore/airWindows/effects/TPDFWideProc.cpp"
#include "src/sfxCore/airWindows/effects/TransDesk.cpp"
#include "src/sfxCore/airWindows/effects/TransDeskProc.cpp"
#include "src/sfxCore/airWindows/effects/Tremolo.cpp"
#include "src/sfxCore/airWindows/effects/TremoloProc.cpp"
#include "src/sfxCore/airWindows/effects/TremoSquare.cpp"
#include "src/sfxCore/airWindows/effects/TremoSquareProc.cpp"
#include "src/sfxCore/airWindows/effects/Trianglizer.cpp"
#include "src/sfxCore/airWindows/effects/TrianglizerProc.cpp"
#include "src/sfxCore/airWindows/effects/TripleSpread.cpp"
#include "src/sfxCore/airWindows/effects/TripleSpreadProc.cpp"
#include "src/sfxCore/airWindows/effects/Tube.cpp"
#include "src/sfxCore/airWindows/effects/Tube2.cpp"
#include "src/sfxCore/airWindows/effects/Tube2Proc.cpp"
#include "src/sfxCore/airWindows/effects/TubeDesk.cpp"
#include "src/sfxCore/airWindows/effects/TubeDeskProc.cpp"
#include "src/sfxCore/airWindows/effects/TubeProc.cpp"
#include "src/sfxCore/airWindows/effects/uLawDecode.cpp"
#include "src/sfxCore/airWindows/effects/uLawDecodeProc.cpp"
#include "src/sfxCore/airWindows/effects/uLawEncode.cpp"
#include "src/sfxCore/airWindows/effects/uLawEncodeProc.cpp"
#include "src/sfxCore/airWindows/effects/Ultrasonic.cpp"
#include "src/sfxCore/airWindows/effects/UltrasonicLite.cpp"
#include "src/sfxCore/airWindows/effects/UltrasonicLiteProc.cpp"
#include "src/sfxCore/airWindows/effects/UltrasonicMed.cpp"
#include "src/sfxCore/airWindows/effects/UltrasonicMedProc.cpp"
#include "src/sfxCore/airWindows/effects/UltrasonicProc.cpp"
#include "src/sfxCore/airWindows/effects/UltrasonX.cpp"
#include "src/sfxCore/airWindows/effects/UltrasonXProc.cpp"
#include "src/sfxCore/airWindows/effects/UnBox.cpp"
#include "src/sfxCore/airWindows/effects/UnBoxProc.cpp"
#include "src/sfxCore/airWindows/effects/VariMu.cpp"
#include "src/sfxCore/airWindows/effects/VariMuProc.cpp"
#include "src/sfxCore/airWindows/effects/Verbity.cpp"
#include "src/sfxCore/airWindows/effects/Verbity2.cpp"
#include "src/sfxCore/airWindows/effects/Verbity2Proc.cpp"
#include "src/sfxCore/airWindows/effects/VerbityProc.cpp"
#include "src/sfxCore/airWindows/effects/Vibrato.cpp"
#include "src/sfxCore/airWindows/effects/VibratoProc.cpp"
#include "src/sfxCore/airWindows/effects/VinylDither.cpp"
#include "src/sfxCore/airWindows/effects/VinylDitherProc.cpp"
#include "src/sfxCore/airWindows/effects/VoiceOfTheStarship.cpp"
#include "src/sfxCore/airWindows/effects/VoiceOfTheStarshipProc.cpp"
#include "src/sfxCore/airWindows/effects/VoiceTrick.cpp"
#include "src/sfxCore/airWindows/effects/VoiceTrickProc.cpp"
#include "src/sfxCore/airWindows/effects/Weight.cpp"
#include "src/sfxCore/airWindows/effects/WeightProc.cpp"
#include "src/sfxCore/airWindows/effects/Wider.cpp"
#include "src/sfxCore/airWindows/effects/WiderProc.cpp"
#include "src/sfxCore/airWindows/effects/Wolfbot.cpp"
#include "src/sfxCore/airWindows/effects/WolfbotProc.cpp"
#include "src/sfxCore/airWindows/effects/XBandpass.cpp"
#include "src/sfxCore/airWindows/effects/XBandpassProc.cpp"
#include "src/sfxCore/airWindows/effects/XHighpass.cpp"
#include "src/sfxCore/airWindows/effects/XHighpassProc.cpp"
#include "src/sfxCore/airWindows/effects/XLowpass.cpp"
#include "src/sfxCore/airWindows/effects/XLowpassProc.cpp"
#include "src/sfxCore/airWindows/effects/XNotch.cpp"
#include "src/sfxCore/airWindows/effects/XNotchProc.cpp"
#include "src/sfxCore/airWindows/effects/XRegion.cpp"
#include "src/sfxCore/airWindows/effects/XRegionProc.cpp"
#include "src/sfxCore/airWindows/effects/YBandpass.cpp"
#include "src/sfxCore/airWindows/effects/YBandpassProc.cpp"
#include "src/sfxCore/airWindows/effects/YHighpass.cpp"
#include "src/sfxCore/airWindows/effects/YHighpassProc.cpp"
#include "src/sfxCore/airWindows/effects/YLowpass.cpp"
#include "src/sfxCore/airWindows/effects/YLowpassProc.cpp"
#include "src/sfxCore/airWindows/effects/YNotBandpass.cpp"
#include "src/sfxCore/airWindows/effects/YNotBandpassProc.cpp"
#include "src/sfxCore/airWindows/effects/YNotch.cpp"
#include "src/sfxCore/airWindows/effects/YNotchProc.cpp"
#include "src/sfxCore/airWindows/effects/YNotHighpass.cpp"
#include "src/sfxCore/airWindows/effects/YNotHighpassProc.cpp"
#include "src/sfxCore/airWindows/effects/YNotLowpass.cpp"
#include "src/sfxCore/airWindows/effects/YNotLowpassProc.cpp"
#include "src/sfxCore/airWindows/effects/YNotNotch.cpp"
#include "src/sfxCore/airWindows/effects/YNotNotchProc.cpp"
#include "src/sfxCore/airWindows/effects/ZBandpass.cpp"
#include "src/sfxCore/airWindows/effects/ZBandpass2.cpp"
#include "src/sfxCore/airWindows/effects/ZBandpass2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ZBandpassProc.cpp"
#include "src/sfxCore/airWindows/effects/ZHighpass.cpp"
#include "src/sfxCore/airWindows/effects/ZHighpass2.cpp"
#include "src/sfxCore/airWindows/effects/ZHighpass2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ZHighpassProc.cpp"
#include "src/sfxCore/airWindows/effects/ZLowpass.cpp"
#include "src/sfxCore/airWindows/effects/ZLowpass2.cpp"
#include "src/sfxCore/airWindows/effects/ZLowpass2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ZLowpassProc.cpp"
#include "src/sfxCore/airWindows/effects/ZNotch.cpp"
#include "src/sfxCore/airWindows/effects/ZNotch2.cpp"
#include "src/sfxCore/airWindows/effects/ZNotch2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ZNotchProc.cpp"
#include "src/sfxCore/airWindows/effects/ZOutputStage.cpp"
#include "src/sfxCore/airWindows/effects/ZOutputStageProc.cpp"
#include "src/sfxCore/airWindows/effects/ZRegion.cpp"
#include "src/sfxCore/airWindows/effects/ZRegion2.cpp"
#include "src/sfxCore/airWindows/effects/ZRegion2Proc.cpp"
#include "src/sfxCore/airWindows/effects/ZRegionProc.cpp"
#include "src/sfxCore/airWindows/sfxCoreAirWindows.cpp"
#include "src/sfxCore/neuralAmpModeler/dsp/dsp1.cpp"
#include "src/sfxCore/neuralAmpModeler/dsp/ImpulseResponse.cpp"
#include "src/sfxCore/neuralAmpModeler/dsp/NoiseGate.cpp"
#include "src/sfxCore/neuralAmpModeler/dsp/RecursiveLinearFilter.cpp"
#include "src/sfxCore/neuralAmpModeler/dsp/wav.cpp"
#include "src/sfxCore/neuralAmpModeler/nam/activations.cpp"
#include "src/sfxCore/neuralAmpModeler/nam/convnet.cpp"
#include "src/sfxCore/neuralAmpModeler/nam/dsp.cpp"
#include "src/sfxCore/neuralAmpModeler/nam/get_dsp.cpp"
#include "src/sfxCore/neuralAmpModeler/nam/lstm.cpp"
#include "src/sfxCore/neuralAmpModeler/nam/util.cpp"
#include "src/sfxCore/neuralAmpModeler/nam/wavenet.cpp"
#include "src/sfxCore/neuralAmpModeler/NeuralAmpModeler.cpp"
#include "src/sfxCore/neuralAmpModeler/sfxCoreNeuralAmpModeler.cpp"
#include "src/sfxCore/rakarrack/common/AnalogFilter.cpp"
#include "src/sfxCore/rakarrack/common/Compressor.cpp"
#include "src/sfxCore/rakarrack/common/EffectLFO.cpp"
#include "src/sfxCore/rakarrack/common/Filter.cpp"
#include "src/sfxCore/rakarrack/common/FilterParams.cpp"
#include "src/sfxCore/rakarrack/common/FormantFilter.cpp"
#include "src/sfxCore/rakarrack/common/HarmonicEnhancer.cpp"
#include "src/sfxCore/rakarrack/common/Metronome.cpp"
#include "src/sfxCore/rakarrack/common/RBFilter.cpp"
#include "src/sfxCore/rakarrack/common/Resample.cpp"
#include "src/sfxCore/rakarrack/common/SmbPitchShift.cpp"
#include "src/sfxCore/rakarrack/common/SVFilter.cpp"
#include "src/sfxCore/rakarrack/common/Waveshaper.cpp"
#include "src/sfxCore/rakarrack/effects/Alienwah.cpp"
#include "src/sfxCore/rakarrack/effects/AnalogPhaser.cpp"
#include "src/sfxCore/rakarrack/effects/Arpie.cpp"
#include "src/sfxCore/rakarrack/effects/Chorus.cpp"
#include "src/sfxCore/rakarrack/effects/CoilCrafter.cpp"
#include "src/sfxCore/rakarrack/effects/CompBand.cpp"
#include "src/sfxCore/rakarrack/effects/Convolotron.cpp"
#include "src/sfxCore/rakarrack/effects/Derelict.cpp"
#include "src/sfxCore/rakarrack/effects/DistBand.cpp"
#include "src/sfxCore/rakarrack/effects/Distorsion.cpp"
#include "src/sfxCore/rakarrack/effects/DualFlange.cpp"
#include "src/sfxCore/rakarrack/effects/DynamicFilter.cpp"
#include "src/sfxCore/rakarrack/effects/Echo.cpp"
#include "src/sfxCore/rakarrack/effects/Echotron.cpp"
#include "src/sfxCore/rakarrack/effects/Echoverse.cpp"
#include "src/sfxCore/rakarrack/effects/EQ.cpp"
#include "src/sfxCore/rakarrack/effects/Exciter.cpp"
#include "src/sfxCore/rakarrack/effects/Expander.cpp"
#include "src/sfxCore/rakarrack/effects/Harmonizer.cpp"
#include "src/sfxCore/rakarrack/effects/Looper.cpp"
#include "src/sfxCore/rakarrack/effects/MusicalDelay.cpp"
#include "src/sfxCore/rakarrack/effects/MuTroMojo.cpp"
#include "src/sfxCore/rakarrack/effects/NoiseGate.cpp"
#include "src/sfxCore/rakarrack/effects/OpticalTrem.cpp"
#include "src/sfxCore/rakarrack/effects/Pan.cpp"
#include "src/sfxCore/rakarrack/effects/Phaser.cpp"
#include "src/sfxCore/rakarrack/effects/Reverb.cpp"
#include "src/sfxCore/rakarrack/effects/Reverbtron.cpp"
#include "src/sfxCore/rakarrack/effects/Ring.cpp"
#include "src/sfxCore/rakarrack/effects/Sequence.cpp"
#include "src/sfxCore/rakarrack/effects/ShelfBoost.cpp"
#include "src/sfxCore/rakarrack/effects/Shifter.cpp"
#include "src/sfxCore/rakarrack/effects/Shuffle.cpp"
#include "src/sfxCore/rakarrack/effects/StereoHarm.cpp"
#include "src/sfxCore/rakarrack/effects/StompBox.cpp"
#include "src/sfxCore/rakarrack/effects/Sustainer.cpp"
#include "src/sfxCore/rakarrack/effects/Synthfilter.cpp"
#include "src/sfxCore/rakarrack/effects/Valve.cpp"
#include "src/sfxCore/rakarrack/effects/VaryBand.cpp"
#include "src/sfxCore/rakarrack/effects/Vibe.cpp"
#include "src/sfxCore/rakarrack/effects/Vocoder.cpp"
#include "src/sfxPluginClap.cpp"
#include "src/sfxPluginLv2.cpp"
#include "src/sfxPluginVst.cpp"
#include "src/sfxPluginVst3.cpp"
#include "src/shader.cpp"
#include "src/shred.cpp"
#include "src/skybox.cpp"
#include "src/sng.cpp"
#include "src/song.cpp"
#include "src/sound.cpp"
#include "src/spirv.cpp"
#include "src/spotify.cpp"
#include "src/stats.cpp"
#include "src/string_.cpp"
#include "src/test.cpp"
#include "src/texture.cpp"
#include "src/tones.cpp"
#include "src/tunerThread.cpp"
#include "src/ui.cpp"
#include "src/wem.cpp"
#include "src/window.cpp"
#include "src/xr.cpp"

#include "src/stretcher.cpp" // something is strange with rubberband. If this is not the last cpp file the exe will crash on exit when building with buildWindows.bat
