// SPDX-License-Identifier: Unlicense

#include "manifest.h"

#include "data.h"
#include "global.h"
#include "helper.h"
#include "json.h"

#include <string.h>

static void readAttribute(Json::object_element* it, Manifest::Info& manifestInfo, std::u8string& artistName, std::u8string& songName, std::u8string& albumName, i32& songYear, TimeNS& songLengthNS)
{
  if (0 == strcmp(it->name->string, "AlbumArt"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.albumArt = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "AlbumName"))
  {
    ASSERT(it->value->type == Json::type_string);
    Json::string* string = (Json::string*)it->value->payload;
    ASSERT(albumName.size() == 0 || albumName == reinterpret_cast<const char8_t*>(string->string));
    albumName = reinterpret_cast<const char8_t*>(string->string);
    return;
  }
  if (0 == strcmp(it->name->string, "AlbumNameSort"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.albumNameSort = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "ArrangementName"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.arrangementName = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "ArtistName"))
  {
    ASSERT(it->value->type == Json::type_string);
    Json::string* string = (Json::string*)it->value->payload;
    ASSERT(artistName.size() == 0 || artistName == reinterpret_cast<const char8_t*>(string->string));
    artistName = reinterpret_cast<const char8_t*>(string->string);
    return;
  }
  if (0 == strcmp(it->name->string, "ArtistNameSort"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.artistNameSort = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "BassPick"))
  {
    ASSERT(it->value->type == Json::type_number);
    Json::number* number = (Json::number*)it->value->payload;
    manifestInfo.bassPick = bool(atoi(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "CapoFret"))
  {
    ASSERT(it->value->type == Json::type_number);
    Json::number* number = (Json::number*)it->value->payload;
    ASSERT(std::string(number->number).ends_with(".0")); // no idea why capo is a float
    manifestInfo.capoFret = i8(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "CentOffset"))
  {
    ASSERT(it->value->type == Json::type_number);
    Json::number* number = (Json::number*)it->value->payload;
    manifestInfo.centOffset = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "DLC"))
  {
    //ASSERT(it->value->type == Json::type_true || it->value->type == Json::type_false);
    //manifestInfo.dLC = it->value->type == Json::type_true;
    return;
  }
  if (0 == strcmp(it->name->string, "DLCKey"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.dLCKey = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "DNA_Chords"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.dNA_Chords = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "DNA_Riffs"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.dNA_Riffs = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "DNA_Solo"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.dNA_Solo = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "EasyMastery"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.easyMastery = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "LeaderboardChallengeRating"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.leaderboardChallengeRating = atoi(number->number);
    return;
  }
  if (0 == strcmp(it->name->string, "ManifestUrn"))
  {
    ASSERT(it->value->type == Json::type_string);
    Json::string* string = (Json::string*)it->value->payload;
    manifestInfo.manifestUrn = reinterpret_cast<const char8_t*>(string->string);
    return;
  }
  if (0 == strcmp(it->name->string, "MasterID_RDV"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.masterID_RDV = atoi(number->number);
    return;
  }
  if (0 == strcmp(it->name->string, "Metronome"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.metronome = atoi(number->number);
    return;
  }
  if (0 == strcmp(it->name->string, "MediumMastery"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.mediumMastery = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "NotesEasy"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.notesEasy = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "NotesHard"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.notesHard = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "NotesMedium"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.notesMedium = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "Representative"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.representative = atoi(number->number);
    return;
  }
  if (0 == strcmp(it->name->string, "RouteMask"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.routeMask = atoi(number->number);
    return;
  }
  if (0 == strcmp(it->name->string, "Shipping"))
  {
    //ASSERT(it->value->type == Json::type_true || it->value->type == Json::type_false);
    //manifestInfo.shipping = it->value->type == Json::type_true;
    return;
  }
  if (0 == strcmp(it->name->string, "SKU"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.sKU = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "SongDiffEasy"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.notesMedium = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "SongDiffHard"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.notesMedium = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "SongDiffMed"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.notesMedium = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "SongDifficulty"))
  {
    //ASSERT(it->value->type == Json::type_number);
    //Json::number* number = (Json::number*)it->value->payload;
    //manifestInfo.notesMedium = f32(atof(number->number));
    return;
  }
  if (0 == strcmp(it->name->string, "SongKey"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.songKey = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "SongLength"))
  {
    ASSERT(it->value->type == Json::type_number);
    Json::number* number = (Json::number*)it->value->payload;
    songLengthNS = timeNS_From_String(reinterpret_cast<const char8_t*>(number->number));
    //manifestInfo.songLength = TimeNS_To_Seconds(songLengthNS);
    return;
  }
  if (0 == strcmp(it->name->string, "SongName"))
  {
    ASSERT(it->value->type == Json::type_string);
    Json::string* string = (Json::string*)it->value->payload;
    ASSERT(songName.size() == 0 || songName == reinterpret_cast<const char8_t*>(string->string));
    songName = reinterpret_cast<const char8_t*>(string->string);
    return;
  }
  if (0 == strcmp(it->name->string, "SongNameSort"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.songNameSort = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "SongYear"))
  {
    ASSERT(it->value->type == Json::type_number);
    Json::number* number = (Json::number*)it->value->payload;
    ASSERT(songYear == 0 || songYear == atoi(number->number));
    songYear = atoi(number->number);
    return;
  }
  if (0 == strcmp(it->name->string, "JapaneseSongName"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.japaneseSongName = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "JapaneseArtist"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.japaneseArtist = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "JapaneseArtistName"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //manifestInfo.japaneseArtistName = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "Tuning"))
  {
    Json::value* tuning_value = it->value;
    ASSERT(tuning_value->type == Json::type_object);

    Json::object* tuning_o = (Json::object*)tuning_value->payload;

    Json::object_element* string = tuning_o->start;
    {
      ASSERT(0 == strcmp(string->name->string, "string0"));
      ASSERT(string->value->type == Json::type_number);
      Json::number* number = (Json::number*)string->value->payload;
      manifestInfo.tuning.string[5] = i8(atoi(number->number));
    }
    string = string->next;
    {
      ASSERT(0 == strcmp(string->name->string, "string1"));
      ASSERT(string->value->type == Json::type_number);
      Json::number* number = (Json::number*)string->value->payload;
      manifestInfo.tuning.string[4] = i8(atoi(number->number));
    }
    string = string->next;
    {
      ASSERT(0 == strcmp(string->name->string, "string2"));
      ASSERT(string->value->type == Json::type_number);
      Json::number* number = (Json::number*)string->value->payload;
      manifestInfo.tuning.string[3] = i8(atoi(number->number));
    }
    string = string->next;
    {
      ASSERT(0 == strcmp(string->name->string, "string3"));
      ASSERT(string->value->type == Json::type_number);
      Json::number* number = (Json::number*)string->value->payload;
      manifestInfo.tuning.string[2] = i8(atoi(number->number));
    }
    string = string->next;
    {
      ASSERT(0 == strcmp(string->name->string, "string4"));
      ASSERT(string->value->type == Json::type_number);
      Json::number* number = (Json::number*)string->value->payload;
      manifestInfo.tuning.string[1] = i8(atoi(number->number));
    }
    string = string->next;
    {
      ASSERT(0 == strcmp(string->name->string, "string5"));
      ASSERT(string->value->type == Json::type_number);
      Json::number* number = (Json::number*)string->value->payload;
      manifestInfo.tuning.string[0] = i8(atoi(number->number));
    }

    return;
  }
  if (0 == strcmp(it->name->string, "PersistentID"))
  {
    //ASSERT(it->value->type == Json::type_string);
    //Json::string* string = (Json::string*)it->value->payload;
    //ASSERT(string->string_size == 32);
    //manifestInfo.persistentID = string->string;
    return;
  }
  if (0 == strcmp(it->name->string, "JapaneseVocal"))
  {
    //ASSERT(it->value->type == Json::type_true || it->value->type == Json::type_false);
    //manifestInfo.japaneseVocal = it->value->type == Json::type_true;
    return;
  }

  ASSERT(false);
}

std::vector<Manifest::Info> Manifest::readPsarcHsan(const std::vector<u8>& hsanData, std::u8string& artistName, std::u8string& songName, std::u8string& albumName, i32& songYear, TimeNS& songLength)
{
  std::vector<Manifest::Info> manifestInfos;

  Json::value* root = Json::parse(hsanData.data(), hsanData.size());
  ASSERT(root != nullptr);
  ASSERT(root->type == Json::type_object);

  Json::object* object = (Json::object*)root->payload;
  ASSERT(object->length >= 2);

  Json::object_element* entries = object->start;

  ASSERT(0 == strcmp(entries->name->string, "Entries"));

  Json::value* entries_value = entries->value;
  ASSERT(entries_value->type == Json::type_object);

  Json::object* id_o = (Json::object*)entries_value->payload;

  Json::object_element* id = id_o->start;
  do
  {
    ASSERT(id->name->string_size == 32);

    Manifest::Info entry;

    Json::value* id_value = id->value;

    ASSERT(id_value->type == Json::type_object);

    Json::object* attributes_o = (Json::object*)id_value->payload;
    ASSERT(attributes_o->length == 1);

    {
      Json::object_element* attributes = attributes_o->start;
      ASSERT(0 == strcmp(attributes->name->string, "Attributes"));

      {
        Json::value* attributes_value = attributes->value;
        ASSERT(attributes_value->type == Json::type_object);

        Json::object* attribute = (Json::object*)attributes_value->payload;

        Json::object_element* it = attribute->start;
        do
        {
          readAttribute(it, entry, artistName, songName, albumName, songYear, songLength);
          it = it->next;
        } while (it != nullptr);
      }

      if (!entry.manifestUrn.ends_with(u8"_vocals") && !entry.manifestUrn.ends_with(u8"_jvocals"))
      {
        manifestInfos.push_back(entry);
      }
    }
    id = id->next;
  } while (id != nullptr);

  delete root;

  return manifestInfos;
}
