// SPDX-License-Identifier: Unlicense

#include "sfx.h"

#ifdef SHR3D_SFX

#include "base64.h"
#include "global.h"
#include "string_.h"

#ifdef SHR3D_SFX_CORE
#include "sfxCore.h"
#endif // SHR3D_SFX_CORE

#ifdef SHR3D_SFX_PLUGIN_CLAP
#include "sfxPluginClap.h"
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_VST
#include "sfxPluginVst.h"
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
#include "sfxPluginVst3.h"
#endif // SHR3D_SFX_PLUGIN_VST3

std::vector<std::u8string> Sfx::names{ u8"...",
#ifdef SHR3D_SFX_CORE_NEURALAMPMODELER
  u8"Neural Amp Modeler",
#endif // SHR3D_SFX_CORE_NEURALAMPMODELER
#ifdef SHR3D_SFX_CORE_HEXFIN
  u8"Hexfin",
#endif // SHR3D_SFX_CORE_HEXFIN

#ifdef SHR3D_SFX_CORE_AIRWINDOWS
  u8"Acceleration",
  u8"Acceleration2",
  u8"ADClip7",
  u8"ADClip8",
  u8"ADT",
  u8"Air",
  u8"Air2",
  u8"Air3",
  u8"Apicolypse",
  u8"AQuickVoiceClip",
  u8"AtmosphereBuss",
  u8"AtmosphereChannel",
  u8"Aura",
  u8"AutoPan",
  u8"Average",
  u8"AverMatrix",
  u8"Balanced",
  u8"BassAmp",
  u8"BassDrive",
  u8"BassKit",
  u8"Baxandall",
  u8"Baxandall2",
  u8"Beam",
  u8"BigAmp",
  u8"Biquad",
  u8"Biquad2",
  u8"BiquadDouble",
  u8"BiquadHiLo",
  u8"BiquadNonLin",
  u8"BiquadOneHalf",
  u8"BiquadPlus",
  u8"BiquadStack",
  u8"BiquadTriple",
  u8"Bite",
  u8"BitGlitter",
  u8"BitShiftGain",
  u8"BitShiftPan",
  u8"BlockParty",
  u8"BrassRider",
  u8"BrightAmbience",
  u8"BrightAmbience2",
  u8"BrightAmbience3",
  u8"BuildATPDF",
  u8"BussColors4",
  u8"ButterComp",
  u8"ButterComp2",
  u8"C5RawBuss",
  u8"C5RawChannel",
  u8"Cabs",
  u8"Calibre",
  u8"Capacitor",
  u8"Capacitor2",
  u8"Chamber",
  u8"Chamber2",
  u8"Channel4",
  u8"Channel5",
  u8"Channel6",
  u8"Channel7",
  u8"Channel8",
  u8"Channel9",
  u8"Chorus",
  u8"ChorusEnsemble",
  u8"ChromeOxide",
  u8"Cider",
  u8"ClearCoat",
  u8"ClipOnly2",
  u8"ClipSoftly",
  u8"CloudCoat",
  u8"Coils",
  u8"Coils2",
  u8"Cojones",
  u8"Compresaturator",
  u8"Console0Buss",
  u8"Console0Channel",
  u8"Console4Buss",
  u8"Console4Channel",
  u8"Console5Buss",
  u8"Console5Channel",
  u8"Console5DarkCh",
  u8"Console6Buss",
  u8"Console6Channel",
  u8"Console7Buss",
  u8"Console7Cascade",
  u8"Console7Channel",
  u8"Console7Crunch",
  u8"Console8BussHype",
  u8"Console8BussIn",
  u8"Console8BussOut",
  u8"Console8ChannelHype",
  u8"Console8ChannelIn",
  u8"Console8ChannelOut",
  u8"Console8LiteBuss",
  u8"Console8LiteChannel",
  u8"Console8SubHype",
  u8"Console8SubIn",
  u8"Console8SubOut",
  u8"Console9Buss",
  u8"Console9Channel",
  u8"ConsoleLABuss",
  u8"ConsoleLAChannel",
  u8"ConsoleMCBuss",
  u8"ConsoleMCChannel",
  u8"ConsoleMDBuss",
  u8"ConsoleMDChannel",
  u8"ConsoleXBuss",
  u8"ConsoleXChannel",
  u8"ContentHideD",
  u8"CreamCoat",
  u8"Creature",
  u8"CrickBass",
  u8"CrunchCoat",
  u8"CrunchyGrooveWear",
  u8"Crystal",
  u8"CStrip",
  u8"CStrip2",
  u8"curve",
  u8"Dark",
  u8"DarkNoise",
  u8"DCVoltage",
  u8"DeBess",
  u8"Deckwrecka",
  u8"DeEss",
  u8"DeHiss",
  u8"Density",
  u8"Density2",
  u8"DeRez",
  u8"DeRez2",
  u8"DeRez3",
  u8"Desk",
  u8"Desk4",
  u8"DigitalBlack",
  u8"Dirt",
  u8"Discontinuity",
  u8"Distance",
  u8"Distance2",
  u8"Distance3",
  u8"Distortion",
  u8"Ditherbox",
  u8"DitherFloat",
  u8"DitherMeDiskers",
  u8"DitherMeTimbers",
  u8"Doublelay",
  u8"DoublePaul",
  u8"Drive",
  u8"DrumSlam",
  u8"DubCenter",
  u8"Dubly",
  u8"DubSub",
  u8"DustBunny",
  u8"Dynamics",
  u8"Dyno",
  u8"Edge",
  u8"EdIsDim",
  u8"Elation",
  u8"ElectroHat",
  u8"Energy",
  u8"Energy2",
  u8"Ensemble",
  u8"EQ",
  u8"EveryConsole",
  u8"EverySlew",
  u8"EveryTrim",
  u8"Exciter",
  u8"Facet",
  u8"FathomFive",
  u8"FinalClip",
  u8"FireAmp",
  u8"Flipity",
  u8"Floor",
  u8"Flutter",
  u8"Focus",
  u8"Fracture",
  u8"Fracture2",
  u8"FromTape",
  u8"Galactic",
  u8"Galactic2",
  u8"GalacticVibe",
  u8"Gatelope",
  u8"GlitchShifter",
  u8"GoldenSlew",
  u8"Golem",
  u8"GrindAmp",
  u8"Gringer",
  u8"GrooveWear",
  u8"GuitarConditioner",
  u8"HardVacuum",
  u8"Hermepass",
  u8"HermeTrim",
  u8"HighGlossDither",
  u8"HighImpact",
  u8"Highpass",
  u8"Highpass2",
  u8"Holt",
  u8"Holt2",
  u8"Hombre",
  u8"Huge",
  u8"Hull",
  u8"Hull2",
  u8"Hype",
  u8"Hypersonic",
  u8"HypersonX",
  u8"Infinity",
  u8"Infinity2",
  u8"Inflamer",
  u8"Infrasonic",
  u8"Interstage",
  u8"IronOxide5",
  u8"IronOxideClassic",
  u8"IronOxideClassic2",
  u8"Isolator",
  u8"Isolator2",
  u8"Kalman",
  u8"kCathedral",
  u8"kCathedral2",
  u8"kCathedral3",
  u8"kChamberAR",
  u8"kGuitarHall",
  u8"kPlate140",
  u8"kPlate240",
  u8"kPlateA",
  u8"kPlateB",
  u8"kPlateC",
  u8"kPlateD",
  u8"LeadAmp",
  u8"LeftoMono",
  u8"LilAmp",
  u8"Logical4",
  u8"Loud",
  u8"Lowpass",
  u8"Lowpass2",
  u8"LRFlipTimer",
  u8"Luxor",
  u8"MackEQ",
  u8"Mackity",
  u8"MatrixVerb",
  u8"Melt",
  u8"MidAmp",
  u8"MidSide",
  u8"Mojo",
  u8"Monitoring",
  u8"Monitoring2",
  u8"Monitoring3",
  u8"MoNoam",
  u8"MSFlipTimer",
  u8"MultiBandDistortion",
  u8"MV",
  u8"MV2",
  u8"NaturalizeDither",
  u8"NCSeventeen",
  u8"Neverland",
  u8"Nikola",
  u8"NodeDither",
  u8"Noise",
  u8"NonlinearSpace",
  u8"NotJustAnotherCD",
  u8"NotJustAnotherDither",
  u8"OneCornerClip",
  u8"OrbitKick",
  u8"Overheads",
  u8"Pafnuty",
  u8"Pafnuty2",
  u8"Parametric",
  u8"PaulDither",
  u8"PaulWide",
  u8"PDBuss",
  u8"PDChannel",
  u8"PeaksOnly",
  u8"Pear",
  u8"Pear2",
  u8"PhaseNudge",
  u8"PitchDelay",
  u8"PitchNasty",
  u8"PlatinumSlew",
  u8"PocketVerbs",
  u8"Pockey",
  u8"Pockey2",
  u8"Podcast",
  u8"PodcastDeluxe",
  u8"Point",
  u8"Pop",
  u8"Pop2",
  u8"Pop3",
  u8"PowerSag",
  u8"PowerSag2",
  u8"Precious",
  u8"Preponderant",
  u8"Pressure4",
  u8"Pressure5",
  u8"PurestAir",
  u8"PurestConsole2Buss",
  u8"PurestConsole2Channel",
  u8"PurestConsole3Buss",
  u8"PurestConsole3Channel",
  u8"PurestConsoleBuss",
  u8"PurestConsoleChannel",
  u8"PurestDrive",
  u8"PurestEcho",
  u8"PurestFade",
  u8"PurestGain",
  u8"PurestSquish",
  u8"PurestWarm",
  u8"PurestWarm2",
  u8"Pyewacket",
  u8"RawGlitters",
  u8"RawTimbers",
  u8"Recurve",
  u8"Remap",
  u8"ResEQ",
  u8"ResEQ2",
  u8"Reverb",
  u8"Righteous4",
  u8"RightoMono",
  u8"SampleDelay",
  u8"Shape",
  u8"ShortBuss",
  u8"SideDull",
  u8"Sidepass",
  u8"Silhouette",
  u8"Sinew",
  u8"SingleEndedTriode",
  u8"Slew",
  u8"Slew2",
  u8"Slew3",
  u8"SlewOnly",
  u8"SlewSonic",
  u8"Smooth",
  u8"SoftGate",
  u8"SpatializeDither",
  u8"Spiral",
  u8"Spiral2",
  u8"Srsly",
  u8"Srsly2",
  u8"Srsly3",
  u8"StarChild",
  u8"StarChild2",
  u8"StereoChorus",
  u8"StereoDoubler",
  u8"StereoEnsemble",
  u8"StereoFX",
  u8"Stonefire",
  u8"StoneFireComp",
  u8"StudioTan",
  u8"SubsOnly",
  u8"SubTight",
  u8"Surge",
  u8"SurgeTide",
  u8"Sweeten",
  u8"Swell",
  u8"Tape",
  u8"TapeDelay",
  u8"TapeDelay2",
  u8"TapeDither",
  u8"TapeDust",
  u8"TapeFat",
  u8"Texturize",
  u8"TexturizeMS",
  u8"Thunder",
  u8"ToneSlant",
  u8"ToTape5",
  u8"ToTape6",
  u8"ToVinyl4",
  u8"TPDFDither",
  u8"TPDFWide",
  u8"TransDesk",
  u8"Tremolo",
  u8"TremoSquare",
  u8"Trianglizer",
  u8"TripleSpread",
  u8"Tube",
  u8"Tube2",
  u8"TubeDesk",
  u8"uLawDecode",
  u8"uLawEncode",
  u8"Ultrasonic",
  u8"UltrasonicLite",
  u8"UltrasonicMed",
  u8"UltrasonX",
  u8"UnBox",
  u8"VariMu",
  u8"Verbity",
  u8"Verbity2",
  u8"Vibrato",
  u8"VinylDither",
  u8"VoiceOfTheStarship",
  u8"VoiceTrick",
  u8"Weight",
  u8"Wider",
  u8"Wolfbot",
  u8"XBandpass",
  u8"XHighpass",
  u8"XLowpass",
  u8"XNotch",
  u8"XRegion",
  u8"YBandpass",
  u8"YHighpass",
  u8"YLowpass",
  u8"YNotBandpass",
  u8"YNotch",
  u8"YNotHighpass",
  u8"YNotLowpass",
  u8"YNotNotch",
  u8"ZBandpass",
  u8"ZBandpass2",
  u8"ZHighpass",
  u8"ZHighpass2",
  u8"ZLowpass",
  u8"ZLowpass2",
  u8"ZNotch",
  u8"ZNotch2",
  u8"ZOutputStage",
  u8"ZRegion",
  u8"ZRegion2",
#endif // SHR3D_SFX_CORE_AIRWINDOWS

#ifdef SHR3D_SFX_CORE_RAKARRACK
  u8"Reverb", // works
  u8"Echo", // works
  u8"Chorus", // works
  u8"Flanger", // works
  u8"Phaser", // works
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Overdrive", // fails
  u8"Distorsion", // fails
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"EQ", // fails
  u8"Parametric EQ", // fails
  u8"Cabinet", // fails
  u8"Compressor", // not sure
  u8"WahWah", // works
  u8"AlienWah", // works
  u8"Pan", // works
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
#if 0
  u8"Harmonizer", // fails
#endif
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Musical Delay", // works
  u8"NoiseGate", // works
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Derelict", // fails
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Analog Phaser", // works
  u8"Valve", // works
  u8"DualFlange", // fails
  u8"Ring", // fails
  u8"Exciter", // works
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"DistBand", // fails
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Arpie", // works
  u8"Expander", // works
  u8"Shuffle", // works
  u8"Synthfilter", // works
  u8"VaryBand", // works
#ifdef SHR3D_SFX_CORE_RAKARRACK_CONVOLOTRON
  u8"Convolotron", // works
#endif // SHR3D_SFX_CORE_RAKARRACK_CONVOLOTRON
#if 0
  u8"Looper", // fails
#endif
  u8"MuTroMojo", // works
  u8"Echoverse", // works
  u8"Coil Crafter", // works
  u8"Shelf Boost", // works
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Vocoder", // fails
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Sustainer", // works
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Sequence", // fails
  u8"Shifter", // fails
  u8"StompBox", // fails
  u8"Reverbtron", // fails
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"Echotron", // works
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"StereoHarm", // fails
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  u8"CompBand", // fails
  u8"OpticalTrem", // works
  u8"Vibe", // works
#endif // SHR3D_SFX_CORE_RAKARRACK
};

void Sfx::init()
{
#ifdef SHR3D_SFX_CORE
  SFXCore::init();
#endif // SHR3D_SFX_CORE

#ifdef SHR3D_SFX_PLUGIN_CLAP
  Global::sfxPluginClapIndexBegin = i32(Sfx::names.size());
  SfxPluginClap::init(Sfx::names);
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  Global::sfxPluginLv2IndexBegin = i32(Sfx::names.size());
  SfxPluginLv2::init(Sfx::names);
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  Global::sfxPluginVstIndexBegin = i32(Sfx::names.size());
  SfxPluginVst::init(Sfx::names);
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  Global::sfxPluginVst3IndexBegin = i32(Sfx::names.size());
  SfxPluginVst3::init(Sfx::names);
#endif // SHR3D_SFX_PLUGIN_VST3

  std::u8string_view tunerMidiPluginName;
  const u64 tunerMidiPluginNameSplitPos = Settings::tunerMidiPlugin.find_first_of(',');
  {
    if (tunerMidiPluginNameSplitPos == std::string::npos)
      tunerMidiPluginName = Settings::tunerMidiPlugin;
    else
      tunerMidiPluginName = std::u8string_view(Settings::tunerMidiPlugin.c_str(), tunerMidiPluginNameSplitPos);
  }

  for (i32 i = 0; i < i32(Sfx::names.size()); ++i)
  {
    { // Tuner Plugin
      if (Settings::tunerPlugin == Sfx::names[i])
        Global::tunerPlugin = i;
    }

#ifdef SHR3D_SFX_PLUGIN_VST
    if (tunerMidiPluginName == Sfx::names[i]) // Midi From Audio Plugin
    {
      const char8_t* parametersBase64 = reinterpret_cast<const char8_t*>(&Settings::tunerMidiPlugin.c_str()[tunerMidiPluginNameSplitPos + 1]);
      const u64 parametersBase64Length = Settings::tunerMidiPlugin.size() - tunerMidiPluginNameSplitPos - 1;
      const std::vector<u8> decodedData = Base64::decode(parametersBase64, parametersBase64Length);
      const std::u8string decodedParameters(reinterpret_cast<const char8_t*>(decodedData.data()), decodedData.size());
      loadParameters(i, 0, decodedParameters);
      if (type(i) == SfxType::vst)
        SfxPluginVst::setMidiFromAudioCallback(i - Global::sfxPluginVstIndexBegin, 0, true);
      Global::tunerMidiPlugin = i;
    }
#endif // SHR3D_SFX_PLUGIN_VST
  }
}

#ifdef SHR3D_SFX_PLUGIN_GUI_WINDOW
bool Sfx::hasWindow(i32 index)
{
  ASSERT(Settings::applicationPluginGUIWindow);
  ASSERT(index >= 0);

  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    return SfxPluginClap::hasWindow(index - Global::sfxPluginClapIndexBegin);
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    return SfxPluginLv2::hasWindow(index - Global::sfxPluginLv2IndexBegin);
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    return SfxPluginVst::hasWindow(index - Global::sfxPluginVstIndexBegin);
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    return true /*SfxPluginVst3::hasWindow(index - Global::sfxPluginVst3IndexBegin)*/;
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    return false;
  }
}

void Sfx::openWindow(i32 index, i32 instance)
{
  //ASSERT(Settings::applicationPluginGUIWindow);
  ASSERT(index >= 0);

  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    SfxPluginClap::openWindow(index - Global::sfxPluginClapIndexBegin, instance);
    return;
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    SfxPluginLv2::openWindow(index - Global::sfxPluginClapIndexBegin, instance);
    return;
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    SfxPluginVst::openWindow(index - Global::sfxPluginVstIndexBegin, instance);
    return;
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    SfxPluginVst3::openWindow(index - Global::sfxPluginVst3IndexBegin, instance);
    return;
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    return;
  }
}

Size Sfx::getWindowSize(i32 index, i32 instance)
{
  ASSERT(Settings::applicationPluginGUIWindow);
  ASSERT(index >= 0);

  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    return SfxPluginClap::getWindowSize(index - Global::sfxPluginClapIndexBegin, instance);
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    return SfxPluginLv2::getWindowSize(index - Global::sfxPluginClapIndexBegin);
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    return SfxPluginVst::getWindowSize(index - Global::sfxPluginVstIndexBegin);
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    return SfxPluginVst3::getWindowSize(index - Global::sfxPluginVst3IndexBegin);
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    unreachable();
  }

  ASSERT(false);
  return {};
}

void Sfx::moveWindow(i32 index, i32 instance, i32 x, i32 y)
{
  ASSERT(Settings::applicationPluginGUIWindow);
  ASSERT(index >= 0);

  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    SfxPluginClap::moveWindow(index - Global::sfxPluginClapIndexBegin, instance, x, y);
    return;
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    SfxPluginLv2::moveWindow(index - Global::sfxPluginLv2IndexBegin, x, y);
    return;
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    SfxPluginVst::moveWindow(index - Global::sfxPluginVstIndexBegin, x, y);
    return;
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    SfxPluginVst3::moveWindow(index - Global::sfxPluginVst3IndexBegin, x, y);
    return;
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    unreachable();
  }
}
void Sfx::closeWindow(i32 index, i32 instance)
{
  //ASSERT(Settings::applicationPluginGUIWindow);
  ASSERT(index >= 0);

  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    SfxPluginClap::closeWindow(index - Global::sfxPluginClapIndexBegin, instance);
    return;
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    SfxPluginLv2::closeWindow(index - Global::sfxPluginLv2IndexBegin, instance);
    return;
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    SfxPluginVst::closeWindow(index - Global::sfxPluginVstIndexBegin, instance);
    return;
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    SfxPluginVst3::closeWindow(index - Global::sfxPluginVst3IndexBegin);
    return;
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    break; // TODO: should be unreachable()
  }
}
#endif // SHR3D_SFX_PLUGIN_GUI_WINDOW

#ifdef SHR3D_SFX_PLUGIN
i32 Sfx::numParams(i32 index, i32 instance)
{
  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    return SfxPluginClap::numParams(index - Global::sfxPluginClapIndexBegin, instance);
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    return SfxPluginVst::numParams(index - Global::sfxPluginVstIndexBegin, instance);
#endif // SHR3D_SFX_PLUGIN_VST
  default:
    unreachable();
  };
}

void Sfx::getParameterProperties(i32 index, i32 instance, i32 param, SfxParameterProperties& parameterProperties)
{
  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    return SfxPluginClap::getParameterProperties(index - Global::sfxPluginClapIndexBegin, instance, param, parameterProperties);
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    return SfxPluginVst::getParameterProperties(index - Global::sfxPluginVstIndexBegin, instance, param, parameterProperties);
#endif // SHR3D_SFX_PLUGIN_VST
  default:
    unreachable();
  };
}

f32 Sfx::getParameter(i32 index, i32 instance, i32 param)
{
  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    return SfxPluginClap::getParameter(index - Global::sfxPluginClapIndexBegin, instance, param);
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    return SfxPluginVst::getParameter(index - Global::sfxPluginVstIndexBegin, instance, param);
#endif // SHR3D_SFX_PLUGIN_VST
  default:
    unreachable();
  };
}

void Sfx::setParameter(i32 index, i32 instance, i32 param, f32 value)
{
  switch (type(index))
  {
#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    return SfxPluginClap::setParameter(index - Global::sfxPluginClapIndexBegin, instance, param, value);
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    return SfxPluginVst::setParameter(index - Global::sfxPluginVstIndexBegin, instance, param, value);
#endif // SHR3D_SFX_PLUGIN_VST
  default:
    unreachable();
  };
}
#endif // SHR3D_SFX_PLUGIN

// return true if outBlock was written
bool Sfx::processBlock(i32 index, i32 instance, f32** inBlock, f32** outBlock, i32 blockSize)
{
  ASSERT(index >= 0);

  switch (type(index))
  {
#ifdef SHR3D_SFX_CORE
  case SfxType::core:
    return SFXCore::processBlock(index, instance, inBlock, outBlock, blockSize);
#endif // SHR3D_SFX_CORE

#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    SfxPluginClap::processBlock(index - Global::sfxPluginClapIndexBegin, instance, inBlock, outBlock, blockSize);
    return true;
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    SfxPluginLv2::processBlock(index - Global::sfxPluginLv2IndexBegin, instance, inBlock, outBlock, blockSize);
    return true;
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    SfxPluginVst::processBlock(index - Global::sfxPluginVstIndexBegin, instance, inBlock, outBlock, blockSize);
    return true;
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    SfxPluginVst3::processBlock(index - Global::sfxPluginVst3IndexBegin, instance, inBlock, outBlock, blockSize);
    return true;
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    unreachable();
  }

  ASSERT(false);
  return true;
}

std::u8string Sfx::saveParameters(i32 index, i32 instance)
{
  ASSERT(index > 0);

  switch (type(index))
  {
#ifdef SHR3D_SFX_CORE
  case SfxType::core:
    return SFXCore::saveParameters(index, instance);
#endif // SHR3D_SFX_CORE

#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    return SfxPluginClap::saveParameters(index - Global::sfxPluginClapIndexBegin, instance);
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    return SfxPluginLv2::saveParameters(index - Global::sfxPluginLv2IndexBegin, instance);
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    return SfxPluginVst::saveParameters(index - Global::sfxPluginVstIndexBegin, instance);
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    return SfxPluginVst3::saveParameters(index - Global::sfxPluginVst3IndexBegin, instance);
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    unreachable();
  }

  ASSERT(false);
  return {};
}

void Sfx::loadParameters(i32 index, i32 instance, const std::u8string& parameters)
{
  ASSERT(index > 0);

  switch (type(index))
  {
#ifdef SHR3D_SFX_CORE
  case SfxType::core:
    SFXCore::loadParameters(index, instance, parameters);
    return;
#endif // SHR3D_SFX_CORE

#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    SfxPluginClap::loadParameters(index - Global::sfxPluginClapIndexBegin, instance, parameters);
    return;
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    SfxPluginLv2::loadParameters(index - Global::sfxPluginLv2IndexBegin, instance, parameters);
    return;
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    SfxPluginVst::loadParameters(index - Global::sfxPluginVstIndexBegin, instance, parameters);
    return;
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    SfxPluginVst3::loadParameters(index - Global::sfxPluginVst3IndexBegin, instance, parameters);
    return;
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    unreachable();
  }
}


SfxType Sfx::type(i32 index)
{
#ifdef SHR3D_SFX_PLUGIN_VST3
  if (index >= Global::sfxPluginVst3IndexBegin)
    return SfxType::vst3;
#endif // SHR3D_SFX_PLUGIN_VST3

#ifdef SHR3D_SFX_PLUGIN_VST
  if (index >= Global::sfxPluginVstIndexBegin)
    return SfxType::vst;
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_LV2
  if (index >= Global::pluginLv2IndexBegin)
    return SfxType::lv2;
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_CLAP
  if (index >= Global::sfxPluginClapIndexBegin)
    return SfxType::clap;
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_CORE
  ASSERT(index < to_underlying_(SfxCore::COUNT));
  return SfxType::core;
#endif // SHR3D_SFX_CORE

  unreachable();
}

const char8_t* Sfx::index2typeName(i32 index)
{
  switch (type(index))
  {
#ifdef SHR3D_SFX_CORE
  case SfxType::core:
    return SFXCore::index2typeName(index);
#endif // SHR3D_SFX_CORE

#ifdef SHR3D_SFX_PLUGIN_CLAP
  case SfxType::clap:
    return u8"CLAP";
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  case SfxType::lv2:
    return u8"LV2";
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  case SfxType::vst:
    return u8"VST";
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  case SfxType::vst3:
    return u8"VST3";
#endif // SHR3D_SFX_PLUGIN_VST3
  default:
    unreachable();
  }
}

//SfxType Sfx::typeName2type(const std::u8string& typeName)
//{
//#ifdef SHR3D_SFX_PLUGIN_CLAP
//  if (typeName == u8"CLAP")
//    return SfxType::clap;
//#endif // SHR3D_SFX_PLUGIN_CLAP
//#ifdef SHR3D_SFX_PLUGIN_LV2
//  if (typeName == u8"LV2")
//    return SfxType::lv2;
//#endif // SHR3D_SFX_PLUGIN_LV2
//#ifdef SHR3D_SFX_PLUGIN_VST
//  if (typeName == u8"VST")
//    return SfxType::vst;
//#endif // SHR3D_SFX_PLUGIN_VST
//#ifdef SHR3D_SFX_PLUGIN_VST3
//  if (typeName == u8"VST3")
//    return SfxType::vst3;
//#endif // SHR3D_SFX_PLUGIN_VST3
//#ifdef SHR3D_SFX_CORE
//  return SFXCore::typeName2type(typeName);
//#endif // SHR3D_SFX_CORE
//}

//i32 Sfx::type2MinIndex(SfxType sfxType)
//{
//  switch (sfxType)
//  {
//#ifdef SHR3D_SFX_CORE
//  case SfxType::core:
//    return SFXCore::type2MinIndex(sfxType);
//#endif // SHR3D_SFX_CORE
//#ifdef SHR3D_SFX_PLUGIN_CLAP
//  case SfxType::clap:
//    return Global::sfxPluginClapIndexBegin;
//#endif // SHR3D_SFX_PLUGIN_CLAP
//#ifdef SHR3D_SFX_PLUGIN_LV2
//  case SfxType::lv2:
//    return Global::pluginLv2IndexBegin;
//#endif // SHR3D_SFX_PLUGIN_LV2
//#ifdef SHR3D_SFX_PLUGIN_VST
//  case SfxType::vst:
//    return Global::sfxPluginVstIndexBegin;
//#endif // SHR3D_SFX_PLUGIN_VST
//#ifdef SHR3D_SFX_PLUGIN_VST3
//  case SfxType::vst3:
//    return Global::sfxPluginVst3IndexBegin;
//#endif // SHR3D_SFX_PLUGIN_VST3
//  default:
//    unreachable();
//  }
//}

i32 Sfx::typeName2indexBegin(const std::u8string& typeName)
{
#ifdef SHR3D_SFX_PLUGIN_CLAP
  if (typeName == u8"CLAP")
    return Global::sfxPluginClapIndexBegin;
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  if (typeName == u8"LV2")
    return Global::pluginLv2IndexBegin;
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  if (typeName == u8"VST")
    return Global::sfxPluginVstIndexBegin;
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  if (typeName == u8"VST3")
    return Global::sfxPluginVst3IndexBegin;
#endif // SHR3D_SFX_PLUGIN_VST3

#ifdef SHR3D_SFX_CORE
  return SFXCore::typeName2indexBegin(typeName);
#endif // SHR3D_SFX_CORE
}

i32 Sfx::indexBegin2indexEnd(const i32 minIndex)
{
#ifdef SHR3D_SFX_CORE
  if (minIndex < to_underlying_(SfxCore::COUNT))
    return SFXCore::indexBegin2indexEnd(minIndex);
#endif // SHR3D_SFX_CORE

#ifdef SHR3D_SFX_PLUGIN_CLAP
  if (minIndex == Global::sfxPluginClapIndexBegin)
#ifdef SHR3D_SFX_PLUGIN_LV2
    return Global::pluginLv2IndexBegin - 1;
#else // SHR3D_SFX_PLUGIN_LV2
#ifdef SHR3D_SFX_PLUGIN_VST
    return Global::sfxPluginVstIndexBegin - 1;
#else // SHR3D_SFX_PLUGIN_VST
#ifdef SHR3D_SFX_PLUGIN_VST3
    return Global::sfxPluginVst3IndexBegin - 1;
#else // SHR3D_SFX_PLUGIN_VST3
    return Sfx::names.size() - 1;
#endif // SHR3D_SFX_PLUGIN_VST3
#endif // SHR3D_SFX_PLUGIN_VST
#endif // SHR3D_SFX_PLUGIN_LV2
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  if (minIndex == Global::pluginLv2IndexBegin)
#ifdef SHR3D_SFX_PLUGIN_VST
    return Global::sfxPluginVstIndexBegin - 1;
#else // SHR3D_SFX_PLUGIN_VST
#ifdef SHR3D_SFX_PLUGIN_VST3
    return Global::sfxPluginVst3IndexBegin - 1;
#else // SHR3D_SFX_PLUGIN_VST3
    return Sfx::names.size() - 1;
#endif // SHR3D_SFX_PLUGIN_VST3
#endif // SHR3D_SFX_PLUGIN_VST
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  if (minIndex == Global::sfxPluginVstIndexBegin)
#ifdef SHR3D_SFX_PLUGIN_VST3
    return Global::sfxPluginVst3IndexBegin - 1;
#else // SHR3D_SFX_PLUGIN_VST3
    return Sfx::names.size() - 1;
#endif // SHR3D_SFX_PLUGIN_VST3
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  if (minIndex == Global::sfxPluginVst3IndexBegin)
    return Sfx::names.size() - 1;
#endif // SHR3D_SFX_PLUGIN_VST3

  unreachable();
}

#endif // SHR3D_SFX
