/* ========================================
 *  BuildATPDF - BuildATPDF.h
 *  Copyright (c) 2016 airwindows, Airwindows uses the MIT license
 * ======================================== */

#include "BuildATPDF.h"

#ifdef SHR3D_SFX_CORE_AIRWINDOWS

#include <cmath>
namespace airwinconsolidated::BuildATPDF {

BuildATPDF::BuildATPDF() :
    AirwinConsolidatedBase(kNumParameters)
{
	A = 0.5;
	B = 0.5;
	C = 0.5;
	D = 0.5;
	E = 0.5;
	F = 0.5;
	G = 0.5;
	H = 0.5;
	I = 0.5;
	J = 0.5;
	
	for(int count = 0; count < 11; count++) {bL[count] = 0.0; bR[count] = 0.0; f[count] = 0.0;}

	//this is reset: values being initialized only once. Startup values, whatever they are.
}

BuildATPDF::~BuildATPDF() {}

static float pinParameter(float data)
{
	if (data < 0.0f) return 0.0f;
	if (data > 1.0f) return 1.0f;
	return data;
}

void BuildATPDF::setParameter(VstInt32 index, float value) {
    switch (index) {
        case kParamA: A = value; break;
        case kParamB: B = value; break; //percent. Using this value, it'll be 0-100 everywhere
        case kParamC: C = value; break;
        case kParamD: D = value; break;
        case kParamE: E = value; break;
        case kParamF: F = value; break;
        case kParamG: G = value; break;
        case kParamH: H = value; break;
        case kParamI: I = value; break;
        case kParamJ: J = value; break;
        default: break; // unknown parameter, shouldn't happen!
    }
}

float BuildATPDF::getParameter(VstInt32 index) {
    switch (index) {
        case kParamA: return A; break;
        case kParamB: return B; break;
        case kParamC: return C; break;
        case kParamD: return D; break;
        case kParamE: return E; break;
        case kParamF: return F; break;
        case kParamG: return G; break;
        case kParamH: return H; break;
        case kParamI: return I; break;
        case kParamJ: return J; break;
        default: break; // unknown parameter, shouldn't happen!
    } return 0.0; //we only need to update the relevant name, this is simple to manage
}

void BuildATPDF::getParameterName(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: strncpy (text, "First", kVstMaxParamStrLen); break;
		case kParamB: strncpy (text, "Second", kVstMaxParamStrLen); break;
		case kParamC: strncpy (text, "Third", kVstMaxParamStrLen); break;
		case kParamD: strncpy (text, "Fourth", kVstMaxParamStrLen); break;
		case kParamE: strncpy (text, "Fifth", kVstMaxParamStrLen); break;
		case kParamF: strncpy (text, "Sixth", kVstMaxParamStrLen); break;
		case kParamG: strncpy (text, "Seventh", kVstMaxParamStrLen); break;
		case kParamH: strncpy (text, "Eighth", kVstMaxParamStrLen); break;
		case kParamI: strncpy (text, "Ninth", kVstMaxParamStrLen); break;
		case kParamJ: strncpy (text, "Tenth", kVstMaxParamStrLen); break;
        default: break; // unknown parameter, shouldn't happen!
    } //this is our labels for displaying in the VST host
}

void BuildATPDF::getParameterDisplay(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: float2string ((A*2)-1, text, kVstMaxParamStrLen); break;
        case kParamB: float2string ((B*2)-1, text, kVstMaxParamStrLen); break;
        case kParamC: float2string ((C*2)-1, text, kVstMaxParamStrLen); break;
        case kParamD: float2string ((D*2)-1, text, kVstMaxParamStrLen); break;
        case kParamE: float2string ((E*2)-1, text, kVstMaxParamStrLen); break;
        case kParamF: float2string ((F*2)-1, text, kVstMaxParamStrLen); break;
        case kParamG: float2string ((G*2)-1, text, kVstMaxParamStrLen); break;
        case kParamH: float2string ((H*2)-1, text, kVstMaxParamStrLen); break;
        case kParamI: float2string ((I*2)-1, text, kVstMaxParamStrLen); break;
        case kParamJ: float2string ((J*2)-1, text, kVstMaxParamStrLen); break;
		
        default: break; // unknown parameter, shouldn't happen!
	} //this displays the values and handles 'popups' where it's discrete choices
}

void BuildATPDF::getParameterLabel(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamB: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamC: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamD: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamE: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamF: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamG: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamH: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamI: strncpy (text, " ", kVstMaxParamStrLen); break;
        case kParamJ: strncpy (text, " ", kVstMaxParamStrLen); break;
        default: break; // unknown parameter, shouldn't happen!
    }
}




} // end namespace

#endif // SHR3D_SFX_CORE_AIRWINDOWS
