/* ========================================
 *  Floor - Floor.h
 *  Copyright (c) 2016 airwindows, Airwindows uses the MIT license
 * ======================================== */

#include "Floor.h"

#ifdef SHR3D_SFX_CORE_AIRWINDOWS

#include <cmath>
namespace airwinconsolidated::Floor {

Floor::Floor() :
    AirwinConsolidatedBase(kNumParameters)
{
	A = 0.0;
	B = 0.0;
	C = 1.0;
	
	flip = false;
	iirSample1AL = 0.0;
	iirSample1BL = 0.0;
	iirSample1CL = 0.0;
	iirSample1DL = 0.0;
	iirSample1EL = 0.0;
	iirSample2AL = 0.0;
	iirSample2BL = 0.0;
	iirSample2CL = 0.0;
	iirSample2DL = 0.0;
	iirSample2EL = 0.0;

	iirSample1AR = 0.0;
	iirSample1BR = 0.0;
	iirSample1CR = 0.0;
	iirSample1DR = 0.0;
	iirSample1ER = 0.0;
	iirSample2AR = 0.0;
	iirSample2BR = 0.0;
	iirSample2CR = 0.0;
	iirSample2DR = 0.0;
	iirSample2ER = 0.0;
	
	fpdL = 1.0; while (fpdL < 16386) fpdL = rand()*UINT32_MAX;
	fpdR = 1.0; while (fpdR < 16386) fpdR = rand()*UINT32_MAX;
	//this is reset: values being initialized only once. Startup values, whatever they are.
}

Floor::~Floor() {}

static float pinParameter(float data)
{
	if (data < 0.0f) return 0.0f;
	if (data > 1.0f) return 1.0f;
	return data;
}

void Floor::setParameter(VstInt32 index, float value) {
    switch (index) {
        case kParamA: A = value; break;
        case kParamB: B = value; break;
        case kParamC: C = value; break;
        default: break; // unknown parameter, shouldn't happen!
    }
}

float Floor::getParameter(VstInt32 index) {
    switch (index) {
        case kParamA: return A; break;
        case kParamB: return B; break;
        case kParamC: return C; break;
        default: break; // unknown parameter, shouldn't happen!
    } return 0.0; //we only need to update the relevant name, this is simple to manage
}

void Floor::getParameterName(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: strncpy (text, "Floor", kVstMaxParamStrLen); break;
		case kParamB: strncpy (text, "Drive", kVstMaxParamStrLen); break;
		case kParamC: strncpy (text, "Dry/Wet", kVstMaxParamStrLen); break;
        default: break; // unknown parameter, shouldn't happen!
    } //this is our labels for displaying in the VST host
}

void Floor::getParameterDisplay(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: float2string (A, text, kVstMaxParamStrLen); break;
        case kParamB: float2string (B, text, kVstMaxParamStrLen); break;
        case kParamC: float2string (C, text, kVstMaxParamStrLen); break;
        default: break; // unknown parameter, shouldn't happen!
	} //this displays the values and handles 'popups' where it's discrete choices
}

void Floor::getParameterLabel(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: strncpy (text, "", kVstMaxParamStrLen); break;
        case kParamB: strncpy (text, "", kVstMaxParamStrLen); break;
        case kParamC: strncpy (text, "", kVstMaxParamStrLen); break;
		default: break; // unknown parameter, shouldn't happen!
    }
}




} // end namespace

#endif // SHR3D_SFX_CORE_AIRWINDOWS
