/* ========================================
 *  UltrasonX - UltrasonX.h
 *  Copyright (c) 2016 airwindows, Airwindows uses the MIT license
 * ======================================== */

#include "UltrasonX.h"

#ifdef SHR3D_SFX_CORE_AIRWINDOWS

#include <cmath>
namespace airwinconsolidated::UltrasonX {

UltrasonX::UltrasonX() :
    AirwinConsolidatedBase(kNumParameters)
{
	A = 0.5;
	
	for (int x = 0; x < fix_total; x++) {fixA[x] = 0.0;}
	fixA[fix_reso] = 0.7071; //butterworth Q

	fpdL = 1.0; while (fpdL < 16386) fpdL = rand()*UINT32_MAX;
	fpdR = 1.0; while (fpdR < 16386) fpdR = rand()*UINT32_MAX;
	//this is reset: values being initialized only once. Startup values, whatever they are.
}

UltrasonX::~UltrasonX() {}

static float pinParameter(float data)
{
	if (data < 0.0f) return 0.0f;
	if (data > 1.0f) return 1.0f;
	return data;
}

void UltrasonX::setParameter(VstInt32 index, float value) {
    switch (index) {
        case kParamA: A = value; break;
        default: break; // unknown parameter, shouldn't happen!
    }
}

float UltrasonX::getParameter(VstInt32 index) {
    switch (index) {
        case kParamA: return A; break;
        default: break; // unknown parameter, shouldn't happen!
    } return 0.0; //we only need to update the relevant name, this is simple to manage
}

void UltrasonX::getParameterName(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: strncpy (text, "Q", kVstMaxParamStrLen); break;
        default: break; // unknown parameter, shouldn't happen!
    } //this is our labels for displaying in the VST host
}

void UltrasonX::getParameterDisplay(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: switch((VstInt32)( A * 4.999 )) //0 to almost edge of # of params
		{	case kA: strncpy (text, "Reso A", kVstMaxParamStrLen); break;
			case kB: strncpy (text, "Reso B", kVstMaxParamStrLen); break;
			case kC: strncpy (text, "Reso C", kVstMaxParamStrLen); break;
			case kD: strncpy (text, "Reso D", kVstMaxParamStrLen); break;
			case kE: strncpy (text, "Reso E", kVstMaxParamStrLen); break;
			default: break; // unknown parameter, shouldn't happen!
		} break;			
        default: break; // unknown parameter, shouldn't happen!
	} //this displays the values and handles 'popups' where it's discrete choices
}

void UltrasonX::getParameterLabel(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: strncpy (text, "", kVstMaxParamStrLen); break;
		default: break; // unknown parameter, shouldn't happen!
    }
}




} // end namespace

#endif // SHR3D_SFX_CORE_AIRWINDOWS
