/*
  ZynAddSubFX - a software synthesizer

  Looper.C - Looper effect
  Copyright (C) 2002-2005 Nasca Octavian Paul
  Author: Nasca Octavian Paul

  Modified for rakarrack by Josep Andreu
  Reverse Looper effect by Transmogrifox

  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License (version 2) for more details.

  You should have received a copy of the GNU General Public License (version 2)
  along with this program; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/

#include "Looper.h"

#ifdef SHR3D_SFX_CORE_RAKARRACK

#if 0

Looper::Looper(f32 size)
{
  settempo(120);
  setbar(2);
  ticker.cleanup();

  Srate_Attack_Coeff = 1.0f / (fSAMPLE_RATE * ATTACK);
  maxx_delay = lrintf(fSAMPLE_RATE * size);
  fade = (i32)SAMPLE_RATE / 2;    //1/2 SR fade time available

  ldelay = new f32[maxx_delay];
  rdelay = new f32[maxx_delay];
  t2ldelay = new f32[maxx_delay];
  t2rdelay = new f32[maxx_delay];

  setpreset(0);
  cleanup();
}

void Looper::cleanuppt1()
{
  for (i32 i = 0; i < maxx_delay; i++)
  {
    ldelay[i] = 0.0f;
    rdelay[i] = 0.0f;
  }
}

void Looper::cleanuppt2()
{
  for (i32 i = 0; i < maxx_delay; i++)
  {
    t2ldelay[i] = 0.0f;
    t2rdelay[i] = 0.0f;
  }
}

void Looper::cleanup()
{
  cleanuppt1();
  cleanuppt2();
}

void Looper::initdelays()
{
  if (PT1)
  {
    kl = 0;
    ticker.cleanup();
    dl = maxx_delay;
    rvkl = dl - 1;
    cleanuppt1();
  }

  if (PT2)
  {
    kl2 = 0;
    ticker.cleanup();
    dl2 = maxx_delay;
    rvkl2 = dl2 - 1;
    cleanuppt2();
  }
  Srate_Attack_Coeff = 90.0f / (dl + dl2);   // Set swell time
}

void Looper::processBlock(const f32* const* inBlock, f32** outBlock, const i32 blockSize)
{
  f32 ticktock[PERIOD];
  if ((Pmetro) && (Pplay) && (!Pstop))
    ticker.metronomeout(ticktock, blockSize);

  for (i32 i = 0; i < blockSize; i++)
  {
    if (Pplay && !Pstop)
    {
      if (Precord)
      {
        if ((Prec1) && (PT1))
        {
          ldelay[kl] += pregain1 * inBlock[0][i];
          rdelay[kl] += pregain1 * inBlock[1][i];
        }
        if ((Prec2) && (PT2))
        {
          t2ldelay[kl2] += pregain2 * inBlock[0][i];
          t2rdelay[kl2] += pregain2 * inBlock[1][i];
        }
      }

      if (!Pclear)
      {
        if (++kl >= dl)
          kl = 0;
        rvkl = dl - 1 - kl;
        if (++kl2 >= dl2)
          kl2 = 0;
        rvkl2 = dl2 - 1 - kl2;
        if ((Plink) || (PT1))
          timeposition(kl);
        else
          timeposition(kl2);
      }

      if (Preverse)
      {
        f32 lswell = (f32)(abs(kl - rvkl)) * Srate_Attack_Coeff;
        if (lswell <= PI_)
        {
          lswell = 0.5f * (1.0f - cosf(lswell));  //Clickless transition
          outBlock[0][i] = (fade1 * ldelay[rvkl] + fade2 * t2ldelay[rvkl2]) * lswell;   //Volume ducking near zero crossing.     
        }
        else
        {
          outBlock[0][i] = fade1 * ldelay[rvkl] + fade2 * t2ldelay[rvkl2];
        }

        f32 rswell = (f32)(abs(kl - rvkl)) * Srate_Attack_Coeff;
        if (rswell <= PI_)
        {
          rswell = 0.5f * (1.0f - cosf(rswell));   //Clickless transition 
          outBlock[1][i] = (fade1 * rdelay[rvkl] + fade2 * t2rdelay[rvkl2]) * rswell;  //Volume ducking near zero crossing.
        }
        else
        {
          outBlock[1][i] = fade1 * rdelay[rvkl] + fade2 * t2rdelay[rvkl2];
        }
      }
      else
      {
        outBlock[0][i] = fade1 * ldelay[kl] + fade2 * t2ldelay[kl2];
        outBlock[1][i] = fade1 * rdelay[kl] + fade2 * t2rdelay[kl2];
      }
    }
    else
    {
      outBlock[0][i] = 0.0f;
      outBlock[1][i] = 0.0f;
    }

    if ((Pmetro) && (Pplay) && (!Pstop))
    {
      outBlock[0][i] += ticktock[i] * mvol;  //if you want to hear the metronome in Looper
      outBlock[1][i] += ticktock[i] * mvol;
    }
  }
}

void Looper::setmvol(i32 value)
{
  mvol = 2.0f * (f32)value / 100.0f;
}

void Looper::settempo(i32 value)
{
  Ptempo = value;
  setbar(Pbar);
  ticker.set_tempo(value);
}

i32 Looper::set_len(i32 value)
{
  const i32 x = value / barlen;

  const i32 a1 = abs(value - (x * barlen));
  const i32 a2 = abs(value - ((x + 1) * barlen));

  if (a2 < a1)
    return(((x + 1) * barlen) + 1);

  return (x * barlen + 1);
}

i32 Looper::cal_len(i32 value)
{
  const i32 lenbar = lrintf((f32)value * 60.0f / (f32)Ptempo * fSAMPLE_RATE);

  looper_ts = lenbar / value;
  return lenbar;
  // return(lrintf((f32)value*60.0/(f32)Ptempo*fSAMPLE_RATE));
}

void Looper::timeposition(i32 value)
{
  looper_bar = value / barlen + 1;
  looper_qua = (value % barlen) / looper_ts + 1;
}

void Looper::setbar(i32 value)
{
  Pbar = value;
  switch (value)
  {
  case 0:
    barlen = cal_len(2);
    if (!Pms)
      ticker.set_meter(2);
    return;
  case 1:
    barlen = cal_len(3);
    if (!Pms)
      ticker.set_meter(3);
    return;
  case 2:
    barlen = cal_len(4);
    if (!Pms)
      ticker.set_meter(4);
    return;
  case 3:
    barlen = cal_len(5);
    if (!Pms)
      ticker.set_meter(5);
    return;
  case 4:
    barlen = cal_len(6);
    if (!Pms)
      ticker.set_meter(6);
    return;
  case 5:
    barlen = cal_len(7);
    if (!Pms)
      ticker.set_meter(7);
    return;
  case 6:
    barlen = cal_len(9);
    if (!Pms)
      ticker.set_meter(9);
    return;
  case 7:
    barlen = cal_len(11);
    if (!Pms)
      ticker.set_meter(11);
    return;
  }
  ASSERT(false);
}

void Looper::setfade()
{
  fade1 = (f32)(Pfade1) / 64.0f;
  fade2 = (f32)(Pfade2) / 64.0f;
  pregain1 = 1.015f / (fade1 + 0.015f);      //this is so material being recorded mixes at the same level as what you hear
  pregain2 = 1.015f / (fade2 + 0.015f);      //magic number is ~1/66, keeps record gain from getting too high
  fade1 *= track1gain;
  fade2 *= track2gain;
}

void Looper::setpreset(i32 npreset)
{
  const i32 PRESET_SIZE = 14;
  i32 presets[][PRESET_SIZE] = {
    //Looper 2 seconds
    {64, 0, 1, 0, 1, 0, 64, 1, 0, 1, 64, 1, 0, 0},
    {64, 0, 1, 0, 1, 1, 64, 1, 0, 1, 64, 1, 0, 0}
  };

  for (i32 n = 0; n < PRESET_SIZE; n++)
    loadpreset(n, presets[npreset][n]);
}

void Looper::changepar(i32 npar, i32 value)
{
  switch (npar)
  {
  case 0:
    Pvolume = value;
    outvolume = (f32)Pvolume / 127.0f;
    return;
  case 1:	//Play at current pointer position
    if (Pplay)
    {
      Pplay = 0;
    }
    else
    {
      Pplay = (PT1 || PT2);  //prevents accidental record state when niether track is active
      if (!Pstop)
      {  //if stop was pushed last time, then resume where it left off
        if (PT1)kl = 0;
        if (PT2)kl2 = 0;
        ticker.cleanup();
      }
      Pstop = 0;
    }
    if (Pstop)
    {
      Pstop = 0;
      Pplay = 0;
    }
    Pclear = 0;
    return;
  case 2:	//stop and reset pointer to 0
    if (Pstop)
      Pstop = 0;
    else
      Pstop = 1;
    if (Precord)
    {
      Pstop = 0;
      if ((first_time1 && Prec1) && PT1)
      {
        dl = set_len(kl);
        //printf("dl %d\n",dl); 
        first_time1 = 0;
        if (Plink)
        {
          dl2 = dl;
        }
        kl = 0;
        ticker.cleanup();

      }
      if ((first_time2 && Prec2) && PT2)
      {
        dl2 = set_len(kl2);
        first_time2 = 0;
        if (Plink)
        {
          dl = dl2;
        }
        kl2 = 0;
        ticker.cleanup();
      }

      Precord = 0;
      Pplay = 0;
    }
    return;
  case 3:		//Record at current position.  If first time (clear = true), then set end of loop, "dl"
    if (Precord)
    {
      if ((first_time1 && Prec1) && PT1)
      {
        dl = set_len(kl);
        first_time1 = 0;
        if (Plink)
        {
          dl2 = dl;
        }
        kl = 0;
        ticker.cleanup();
      }
      if ((first_time2 && Prec2) && PT2)
      {
        dl2 = set_len(kl2);
        first_time2 = 0;
        if (Plink)
        {
          dl = dl2;
        }
        kl2 = 0;
        ticker.cleanup();
      }
      Precord = 0;
      Pplay = rplaystate;
      if (Pautoplay)  Pplay = 1;
      Pstop = 0;
    }
    else
    {
      Precord = 1;
      rplaystate = Pplay;
      if (Pautoplay)
      {
        Pplay = 1;
        if ((first_time1 && Prec1) || (first_time2 && Prec2)) ticker.cleanup();
      }
      else Pplay = 0;
    }
    Pstop = 0;
    Pclear = 0;
    return;
  case 4:
    Pclear = 1;    //Clear everything and erase the loop
    if (PT1) first_time1 = 1;
    if (PT2) first_time2 = 1;
    if ((PT1) && (PT2)) Pplay = 0;
    Precord = 0;
    if (PT1)kl = 0;
    if (PT2)kl2 = 0;
    ticker.cleanup();
    initdelays();
    return;
  case 5:
    Preverse = value;		//Playback in reverse
    return;
  case 6:
    Pfade1 = value;
    setfade();
    return;
  case 7:
    if (PT1) {
      PT1 = 0;
    }
    else
    {
      PT1 = 1;
      if (!Plink) {
        if (first_time1)initdelays();
        PT2 = 0;       //if they aren't linked in time, then you want to toggle 1&2
        track2gain = 0.0f;
        kl = 0;
        ticker.cleanup();
        setfade();
      }

    }
    track1gain = (f32)PT1;
    setfade();
    return;
  case 8:
    if (PT2) {
      PT2 = 0;
    }
    else
    {
      PT2 = 1;
      if (!Plink) {
        PT1 = 0;
        if (first_time2)initdelays();
        kl2 = 0;
        ticker.cleanup();
        track1gain = 0.0f;
      }

    }
    track2gain = (f32)PT2;
    setfade();
    return;
  case 9:
    Pautoplay = value;
    return;
  case 10:
    Pfade2 = value;
    setfade();
    return;
  case 11:
    Prec1 = value;
    return;
  case 12:
    Prec2 = value;
    return;
  case 13:
    Plink = value;
    if (Plink)
    {
      if (Prec1) dl2 = dl;
      if (Prec2) dl = dl2;  //if both are true, then it is only a redundant assignment
    }
    return;
  case 14:
    settempo(value);
    return;
  case 15:
    setbar(value);
    return;
  case 16:
    Pmetro = value;
    return;
  case 17:
    Pms = value;
    if (Pms == 0)
      setbar(Pbar);
    if (Pms == 1)
      ticker.set_meter(1);
    if (Pms == 2)
      ticker.set_meter(0);
    return;
  }
  ASSERT(false);
}

i32 Looper::getpar(i32 npar)
{
  switch (npar)
  {
  case 0:
    return Pvolume;
  case 1:
    return Pplay;
  case 2:
    return Pstop;
  case 3:
    return Precord;
  case 4:
    return Pclear;
  case 5:
    return Preverse;
  case 6:
    return Pfade1;
  case 7:
    return PT1;
  case 8:
    return PT2;
  case 9:
    return Pautoplay;
  case 10:
    return Pfade2;
  case 11:
    return Prec1;
  case 12:
    return Prec2;
  case 13:
    return Plink;
  case 14:
    return Ptempo;
  case 15:
    return Pbar;
  case 16:
    return Pmetro;
  case 17:
    return Pms;
  }
  ASSERT(false);
  return 0;
}

void Looper::getstate()
{
  progstate[0] = Pplay;
  progstate[1] = Pstop;
  progstate[2] = Precord;
  progstate[3] = Pclear;
  progstate[4] = PT1;
  progstate[5] = PT2;
}

void Looper::loadpreset(i32 npar, i32 value)
{
  switch (npar)
  {
  case 0:
    Pvolume = value;
    outvolume = (f32)Pvolume / 127.0f;
    break;
  case 1:	//Play at current pointer position
    Pplay = value;
    break;
  case 2:	//stop and reset pointer to 0
    Pstop = value;
    break;
  case 3:		//Record at current position.  If first time (clear = true), then set end of loop, "dl"
    Precord = value;
    break;
  case 4:
    Pclear = 1;    //Clear everything and erase the loop
    initdelays();
    break;
  case 5:
    Preverse = value;		//Playback in reverse
    break;
  case 6:
    Pfade1 = value;
    setfade();
    break;
  case 7:
    PT1 = value;
    track1gain = (f32)PT1;
    setfade();
    break;
  case 8:
    PT2 = value;
    track2gain = (f32)PT2;
    setfade();
    break;
  case 9:
    Pautoplay = value;
    break;
  case 10:
    Pfade2 = value;
    setfade();
    break;
  case 11:
    Prec1 = value;
    break;
  case 12:
    Prec2 = value;
    break;
  case 13:
    Plink = value;
    break;
  default:
    unreachable();
  }

  Pclear = 1;    //Clear everything and erase the loop
  if (PT1) first_time1 = 1;
  if (PT2) first_time2 = 1;
  if ((PT1) && (PT2)) Pplay = 0;
  Precord = 0;
  getstate();
}

#endif

#endif // SHR3D_SFX_CORE_RAKARRACK
