// SPDX-License-Identifier: Unlicense

#ifndef TYPE_H
#define TYPE_H

#include "typedefs.h"

#include <string>
#include <vector>

enum struct FullscreenMode : u8
{
  windowed,
  borderless,
  //fullscreen // not supported. vst plugin ui does not show.
};

enum struct ScalingMode : i8
{
  HorPlus = 0,
  Strech = 1
};

#if !defined(PLATFORM_EMSCRIPTEN) && !defined(PLATFORM_QUEST_3)
enum struct VSyncMode : i8
{
  noVSync = 0,
  VSync = 1,
#ifdef SHR3D_WINDOW_SDL
  AdaptiveVSync = -1
#endif // SHR3D_WINDOW_SDL
};
#endif // !PLATFORM_EMSCRIPTEN && !PLATFORM_QUEST_3

enum struct AudioSystem : u8
{
#ifdef SHR3D_AUDIO_AAUDIO
  AAudio,
#endif // SHR3D_AUDIO_AAUDIO
#ifdef SHR3D_AUDIO_ASIO
  ASIO,
#endif // SHR3D_AUDIO_ASIO
#ifdef SHR3D_AUDIO_JACK
  JACK,
#endif // SHR3D_AUDIO_JACK
#ifdef SHR3D_AUDIO_PIPEWIRE
  PipeWire,
#endif // SHR3D_AUDIO_PIPEWIRE
#ifdef SHR3D_AUDIO_SDL
  SDL,
#endif // SHR3D_AUDIO_SDL
#ifdef SHR3D_AUDIO_SUPERPOWERED
  Superpowered,
#endif // SHR3D_AUDIO_SUPERPOWERED
#ifdef SHR3D_AUDIO_WASAPI
  WASAPI,
#endif // SHR3D_AUDIO_WASAPI
#ifdef SHR3D_AUDIO_WEBAUDIO
  WebAudio,
#endif // SHR3D_AUDIO_WEBAUDIO  
  COUNT
};

#ifdef SHR3D_AUDIO_AAUDIO
enum struct AAudioPerformanceMode
{
    None,
    PowerSaving,
    LowLatency
};
#endif // SHR3D_AUDIO_AAUDIO

enum struct BackupMode : u8
{
  off,
  onceOnStart
};

enum struct InstallMode : u8
{
  showInstaller,
  continueWithoutSaving,
  installed
};

enum struct Instrument : u8
{
  LeadGuitar,
  RhythmGuitar,
  BassGuitar
};

enum struct EndOfSong : u8
{
  doNothing,
  openMenu,
  loopSong
};

enum struct NoteDetectionSource : u8
{
  pickup,
#ifdef SHR3D_MIDI
  midiDevice,
#endif // SHR3D_MIDI
  midiPlugin
};

enum struct NoteShape : u8
{
  hex,
  rect
};

enum struct NoteSymbols : u8
{
  none,
  fretMute,
  hammerOn,
  harmonic,
  palmMute,
  pinchHarmonic,
  pop,
  pullOff,
  slap,
  tap,
  slimX,
  slimEye,
  triangleDown,
  triangleUp,
  wideX,
  wideV,
  wideUpsideV,
  wideOffsetV,
  wideEye,
  sineRight,
  sineLeft,
  COUNT
};

enum struct SongFormat : u8
{
  sng,
  xml,
};

enum struct SongSortMode : u8
{
  unsorted,
  tuning,
  songName,
  artistName,
  albumName,
  year
};

enum struct CameraMode : u8
{
  fixed,
  linear,
  kinematic,
#ifdef SHR3D_OPENXR_PCVR
  xrFixed,
#endif // SHR3D_OPENXR_PCVR
};

#if defined(SHR3D_HUD_DEVELOPMENT) || defined(SHR3D_RENDERER_DEVELOPMENT)
enum struct Renderer : i8
{
  production,
  development
};
#endif // SHR3D_HUD_DEVELOPMENT || defined(SHR3D_RENDERER_DEVELOPMENT

enum struct MidiNote : u8
{
  C_0 = 12,        //    16.352Hz
  C_sharp_0 = 13,  //    17.324Hz
  D_0 = 14,        //    18.354Hz
  D_sharp_0 = 15,  //    19.445Hz
  E_0 = 16,        //    20.602Hz
  F_0 = 17,        //    21.827Hz
  F_sharp_0 = 18,  //    23.125Hz
  G_0 = 19,        //    24.500Hz
  G_sharp_0 = 20,  //    25.957Hz
  A_0 = 21,        //    27.500Hz
  A_sharp_0 = 22,  //    29.135Hz
  B_0 = 23,        //    30.868Hz
  C_1 = 24,        //    32.703Hz
  C_sharp_1 = 25,  //    34.648Hz
  D_1 = 26,        //    36.708Hz
  D_sharp_1 = 27,  //    38.891Hz
  E_1 = 28,        //    41.203Hz
  F_1 = 29,        //    43.654Hz
  F_sharp_1 = 30,  //    46.249Hz
  G_1 = 31,        //    48.999Hz
  G_sharp_1 = 32,  //    51.913Hz
  A_1 = 33,        //    55.000Hz
  A_sharp_1 = 34,  //    58.270Hz
  B_1 = 35,        //    61.735Hz
  C_2 = 36,        //    65.406Hz
  C_sharp_2 = 37,  //    69.296Hz
  D_2 = 38,        //    73.416Hz
  D_sharp_2 = 39,  //    77.782Hz
  E_2 = 40,        //    82.407Hz
  F_2 = 41,        //    87.307Hz
  F_sharp_2 = 42,  //    92.499Hz
  G_2 = 43,        //    97.999Hz
  G_sharp_2 = 44,  //   103.826Hz
  A_2 = 45,        //   110.000Hz
  A_sharp_2 = 46,  //   116.541Hz
  B_2 = 47,        //   123.471Hz
  C_3 = 48,        //   130.813Hz
  C_sharp_3 = 49,  //   138.591Hz
  D_3 = 50,        //   146.832Hz
  D_sharp_3 = 51,  //   155.563Hz
  E_3 = 52,        //   164.814Hz
  F_3 = 53,        //   174.614Hz
  F_sharp_3 = 54,  //   184.997Hz
  G_3 = 55,        //   195.998Hz
  G_sharp_3 = 56,  //   207.652Hz
  A_3 = 57,        //   220.000Hz
  A_sharp_3 = 58,  //   233.082Hz
  B_3 = 59,        //   246.942Hz
  C_4 = 60,        //   261.626Hz
  C_sharp_4 = 61,  //   277.183Hz
  D_4 = 62,        //   293.665Hz
  D_sharp_4 = 63,  //   311.127Hz
  E_4 = 64,        //   329.628Hz
  F_4 = 65,        //   349.228Hz
  F_sharp_4 = 66,  //   369.994Hz
  G_4 = 67,        //   391.995Hz
  G_sharp_4 = 68,  //   415.305Hz
  A_4 = 69,        //   440.000Hz
  A_sharp_4 = 70,  //   466.164Hz
  B_4 = 71,        //   493.883Hz
  C_5 = 72,        //   523.251Hz
  C_sharp_5 = 73,  //   554.365Hz
  D_5 = 74,        //   587.330Hz
  D_sharp_5 = 75,  //   622.254Hz
  E_5 = 76,        //   659.255Hz
  F_5 = 77,        //   698.456Hz
  F_sharp_5 = 78,  //   739.989Hz
  G_5 = 79,        //   783.991Hz
  G_sharp_5 = 80,  //   830.609Hz
  A_5 = 81,        //   880.000Hz
  A_sharp_5 = 82,  //   932.328Hz
  B_5 = 83,        //   987.767Hz
  C_6 = 84,        //  1046.502Hz
  C_sharp_6 = 85,  //  1108.731Hz
  D_6 = 86,        //  1174.659Hz
  D_sharp_6 = 87,  //  1244.508Hz
  E_6 = 88,        //  1318.510Hz
  F_6 = 89,        //  1396.913Hz
  F_sharp_6 = 90,  //  1479.978Hz
  G_6 = 91,        //  1567.982Hz
  G_sharp_6 = 92,  //  1661.219Hz
  A_6 = 93,        //  1760.000Hz
  A_sharp_6 = 94,  //  1864.655Hz
  B_6 = 95,        //  1975.533Hz
  C_7 = 96,        //  2093.005Hz
  C_sharp_7 = 97,  //  2217.461Hz
  D_7 = 98,        //  2349.318Hz
  D_sharp_7 = 99,  //  2489.016Hz
  E_7 = 100,       //  2637.020Hz
  F_7 = 101,       //  2793.826Hz
  F_sharp_7 = 102, //  2959.955Hz
  G_7 = 103,       //  3135.963Hz
  G_sharp_7 = 104, //  3322.438Hz
  A_7 = 105,       //  3520.000Hz
  A_sharp_7 = 106, //  3729.310Hz
  B_7 = 107,       //  3951.066Hz
  C_8 = 108,       //  4186.009Hz
  C_sharp_8 = 109, //  4434.922Hz
  D_8 = 110,       //  4698.636Hz
  D_sharp_8 = 111, //  4978.032Hz
  E_8 = 112,       //  5274.041Hz
  F_8 = 113,       //  5587.652Hz
  F_sharp_8 = 114, //  5919.911Hz
  G_8 = 115,       //  6271.927Hz
  G_sharp_8 = 116, //  6644.875Hz
  A_8 = 117,       //  7040.000Hz
  A_sharp_8 = 118, //  7458.620Hz
  B_8 = 119,       //  7902.133Hz
  C_9 = 120,       //  8372.018Hz
  C_sharp_9 = 121, //  8869.844Hz
  D_9 = 122,       //  9397.273Hz
  D_sharp_9 = 123, //  9956.063Hz
  E_9 = 124,       // 10548.080Hz
  F_9 = 125,       // 11175.300Hz
  F_sharp_9 = 126, // 11839.820Hz
  G_9 = 127,       // 12543.850Hz
  COUNT = 128
};

#ifdef SHR3D_MIDI
enum struct MidiNoteMode : u8
{
  press,
  toggle,
  analog
};
#endif // SHR3D_MIDI

enum struct MetronomeSide : u8
{
  left,
  right,
  stereo
};

enum struct MusicDoubleBufferStatus : u8
{
  inital = 0b0,
  DataInNonStreched0 = 0b10,
  DataInNonStreched1 = 0b11,
  DataInNonStreched0Delete1 = 0b100,
  DataInNonStreched1Delete0 = 0b101,
};

//enum struct Sfx
//{
//#ifdef SHR3D_SFX_CORE_NEURALAMPMODELER
//  SfxCoreNeuralAmpModeler,
//#endif // SHR3D_SFX_CORE_NEURALAMPMODELER
//#ifdef SHR3D_SFX_CORE_RAKARRACK
//  SfxCoreNeuralRakasrrack,
//#endif // SHR3D_SFX_CORE_RAKARRACK
//#ifdef SHR3D_SFX_CORE_AIRWINDOWS
//  SfxCoreNeuralAirWindows,
//#endif // SHR3D_SFX_CORE_AIRWINDOWS
//#ifdef SHR3D_SFX_PLUGIN_CLAP
//  SfxPluginClap,
//#endif // SHR3D_SFX_PLUGIN_CLAP
//#ifdef SHR3D_SFX_PLUGIN_LV2
//  SfxPluginLv2,
//#endif // SHR3D_SFX_PLUGIN_LV2
//#ifdef SHR3D_SFX_PLUGIN_VST
//  SfxPluginVst,
//#endif // SHR3D_SFX_PLUGIN_VST
//#ifdef SHR3D_SFX_PLUGIN_VST3
//  SfxPluginVst3
//#endif // SHR3D_SFX_PLUGIN_VST3
//};

enum struct SfxType : i8
{
#ifdef SHR3D_SFX_CORE
  core,
#endif // SHR3D_SFX_CORE

#ifdef SHR3D_SFX_PLUGIN_CLAP
  clap,
#endif // SHR3D_SFX_PLUGIN_CLAP

#ifdef SHR3D_SFX_PLUGIN_LV2
  lv2,
#endif // SHR3D_SFX_PLUGIN_LV2

#ifdef SHR3D_SFX_PLUGIN_VST
  vst,
#endif // SHR3D_SFX_PLUGIN_VST

#ifdef SHR3D_SFX_PLUGIN_VST3
  vst3
#endif // SHR3D_SFX_PLUGIN_VST3
};

#ifdef SHR3D_SFX_CORE
enum struct SfxCore
{
  EMPTY,
#ifdef SHR3D_SFX_CORE_NEURALAMPMODELER
  NeuralAmpModeler,
#endif // SHR3D_SFX_CORE_NEURALAMPMODELER
#ifdef SHR3D_SFX_CORE_HEXFIN
  Tuner,
#endif // SHR3D_SFX_CORE_HEXFIN

#if defined (SHR3D_SFX_CORE_NEURALAMPMODELER) || defined (SHR3D_SFX_CORE_HEXFIN)
  NOTYPE_BEGIN = 
#ifdef SHR3D_SFX_CORE_NEURALAMPMODELER
  NeuralAmpModeler,
#else // SHR3D_SFX_CORE_NEURALAMPMODELER
#ifdef SHR3D_SFX_CORE_HEXFIN
  Tuner,
#endif // SHR3D_SFX_CORE_HEXFIN
#endif // SHR3D_SFX_CORE_NEURALAMPMODELER
  NOTYPE_END =
#ifdef SHR3D_SFX_CORE_HEXFIN
  Tuner,
#else // SHR3D_SFX_CORE_HEXFIN
#ifdef SHR3D_SFX_CORE_NEURALAMPMODELER
  NeuralAmpModeler,
#endif // SHR3D_SFX_CORE_NEURALAMPMODELER
#endif // SHR3D_SFX_CORE_HEXFIN
#endif // SHR3D_SFX_CORE_NEURALAMPMODELER || SHR3D_SFX_CORE_HEXFIN

#ifdef SHR3D_SFX_CORE_AIRWINDOWS
  AirWindows_Acceleration,
  AirWindows_Acceleration2,
  AirWindows_ADClip7,
  AirWindows_ADClip8,
  AirWindows_ADT,
  AirWindows_Air,
  AirWindows_Air2,
  AirWindows_Air3,
  AirWindows_Apicolypse,
  AirWindows_AQuickVoiceClip,
  AirWindows_AtmosphereBuss,
  AirWindows_AtmosphereChannel,
  AirWindows_Aura,
  AirWindows_AutoPan,
  AirWindows_Average,
  AirWindows_AverMatrix,
  AirWindows_Balanced,
  AirWindows_BassAmp,
  AirWindows_BassDrive,
  AirWindows_BassKit,
  AirWindows_Baxandall,
  AirWindows_Baxandall2,
  AirWindows_Beam,
  AirWindows_BigAmp,
  AirWindows_Biquad,
  AirWindows_Biquad2,
  AirWindows_BiquadDouble,
  AirWindows_BiquadHiLo,
  AirWindows_BiquadNonLin,
  AirWindows_BiquadOneHalf,
  AirWindows_BiquadPlus,
  AirWindows_BiquadStack,
  AirWindows_BiquadTriple,
  AirWindows_Bite,
  AirWindows_BitGlitter,
  AirWindows_BitShiftGain,
  AirWindows_BitShiftPan,
  AirWindows_BlockParty,
  AirWindows_BrassRider,
  AirWindows_BrightAmbience,
  AirWindows_BrightAmbience2,
  AirWindows_BrightAmbience3,
  AirWindows_BuildATPDF,
  AirWindows_BussColors4,
  AirWindows_ButterComp,
  AirWindows_ButterComp2,
  AirWindows_C5RawBuss,
  AirWindows_C5RawChannel,
  AirWindows_Cabs,
  AirWindows_Calibre,
  AirWindows_Capacitor,
  AirWindows_Capacitor2,
  AirWindows_Chamber,
  AirWindows_Chamber2,
  AirWindows_Channel4,
  AirWindows_Channel5,
  AirWindows_Channel6,
  AirWindows_Channel7,
  AirWindows_Channel8,
  AirWindows_Channel9,
  AirWindows_Chorus,
  AirWindows_ChorusEnsemble,
  AirWindows_ChromeOxide,
  AirWindows_Cider,
  AirWindows_ClearCoat,
  AirWindows_ClipOnly2,
  AirWindows_ClipSoftly,
  AirWindows_CloudCoat,
  AirWindows_Coils,
  AirWindows_Coils2,
  AirWindows_Cojones,
  AirWindows_Compresaturator,
  AirWindows_Console0Buss,
  AirWindows_Console0Channel,
  AirWindows_Console4Buss,
  AirWindows_Console4Channel,
  AirWindows_Console5Buss,
  AirWindows_Console5Channel,
  AirWindows_Console5DarkCh,
  AirWindows_Console6Buss,
  AirWindows_Console6Channel,
  AirWindows_Console7Buss,
  AirWindows_Console7Cascade,
  AirWindows_Console7Channel,
  AirWindows_Console7Crunch,
  AirWindows_Console8BussHype,
  AirWindows_Console8BussIn,
  AirWindows_Console8BussOut,
  AirWindows_Console8ChannelHype,
  AirWindows_Console8ChannelIn,
  AirWindows_Console8ChannelOut,
  AirWindows_Console8LiteBuss,
  AirWindows_Console8LiteChannel,
  AirWindows_Console8SubHype,
  AirWindows_Console8SubIn,
  AirWindows_Console8SubOut,
  AirWindows_Console9Buss,
  AirWindows_Console9Channel,
  AirWindows_ConsoleLABuss,
  AirWindows_ConsoleLAChannel,
  AirWindows_ConsoleMCBuss,
  AirWindows_ConsoleMCChannel,
  AirWindows_ConsoleMDBuss,
  AirWindows_ConsoleMDChannel,
  AirWindows_ConsoleXBuss,
  AirWindows_ConsoleXChannel,
  AirWindows_ContentHideD,
  AirWindows_CreamCoat,
  AirWindows_Creature,
  AirWindows_CrickBass,
  AirWindows_CrunchCoat,
  AirWindows_CrunchyGrooveWear,
  AirWindows_Crystal,
  AirWindows_CStrip,
  AirWindows_CStrip2,
  AirWindows_curve,
  AirWindows_Dark,
  AirWindows_DarkNoise,
  AirWindows_DCVoltage,
  AirWindows_DeBess,
  AirWindows_Deckwrecka,
  AirWindows_DeEss,
  AirWindows_DeHiss,
  AirWindows_Density,
  AirWindows_Density2,
  AirWindows_DeRez,
  AirWindows_DeRez2,
  AirWindows_DeRez3,
  AirWindows_Desk,
  AirWindows_Desk4,
  AirWindows_DigitalBlack,
  AirWindows_Dirt,
  AirWindows_Discontinuity,
  AirWindows_Distance,
  AirWindows_Distance2,
  AirWindows_Distance3,
  AirWindows_Distortion,
  AirWindows_Ditherbox,
  AirWindows_DitherFloat,
  AirWindows_DitherMeDiskers,
  AirWindows_DitherMeTimbers,
  AirWindows_Doublelay,
  AirWindows_DoublePaul,
  AirWindows_Drive,
  AirWindows_DrumSlam,
  AirWindows_DubCenter,
  AirWindows_Dubly,
  AirWindows_DubSub,
  AirWindows_DustBunny,
  AirWindows_Dynamics,
  AirWindows_Dyno,
  AirWindows_Edge,
  AirWindows_EdIsDim,
  AirWindows_Elation,
  AirWindows_ElectroHat,
  AirWindows_Energy,
  AirWindows_Energy2,
  AirWindows_Ensemble,
  AirWindows_EQ,
  AirWindows_EveryConsole,
  AirWindows_EverySlew,
  AirWindows_EveryTrim,
  AirWindows_Exciter,
  AirWindows_Facet,
  AirWindows_FathomFive,
  AirWindows_FinalClip,
  AirWindows_FireAmp,
  AirWindows_Flipity,
  AirWindows_Floor,
  AirWindows_Flutter,
  AirWindows_Focus,
  AirWindows_Fracture,
  AirWindows_Fracture2,
  AirWindows_FromTape,
  AirWindows_Galactic,
  AirWindows_Galactic2,
  AirWindows_GalacticVibe,
  AirWindows_Gatelope,
  AirWindows_GlitchShifter,
  AirWindows_GoldenSlew,
  AirWindows_Golem,
  AirWindows_GrindAmp,
  AirWindows_Gringer,
  AirWindows_GrooveWear,
  AirWindows_GuitarConditioner,
  AirWindows_HardVacuum,
  AirWindows_Hermepass,
  AirWindows_HermeTrim,
  AirWindows_HighGlossDither,
  AirWindows_HighImpact,
  AirWindows_Highpass,
  AirWindows_Highpass2,
  AirWindows_Holt,
  AirWindows_Holt2,
  AirWindows_Hombre,
  AirWindows_Huge,
  AirWindows_Hull,
  AirWindows_Hull2,
  AirWindows_Hype,
  AirWindows_Hypersonic,
  AirWindows_HypersonX,
  AirWindows_Infinity,
  AirWindows_Infinity2,
  AirWindows_Inflamer,
  AirWindows_Infrasonic,
  AirWindows_Interstage,
  AirWindows_IronOxide5,
  AirWindows_IronOxideClassic,
  AirWindows_IronOxideClassic2,
  AirWindows_Isolator,
  AirWindows_Isolator2,
  AirWindows_Kalman,
  AirWindows_kCathedral,
  AirWindows_kCathedral2,
  AirWindows_kCathedral3,
  AirWindows_kChamberAR,
  AirWindows_kGuitarHall,
  AirWindows_kPlate140,
  AirWindows_kPlate240,
  AirWindows_kPlateA,
  AirWindows_kPlateB,
  AirWindows_kPlateC,
  AirWindows_kPlateD,
  AirWindows_LeadAmp,
  AirWindows_LeftoMono,
  AirWindows_LilAmp,
  AirWindows_Logical4,
  AirWindows_Loud,
  AirWindows_Lowpass,
  AirWindows_Lowpass2,
  AirWindows_LRFlipTimer,
  AirWindows_Luxor,
  AirWindows_MackEQ,
  AirWindows_Mackity,
  AirWindows_MatrixVerb,
  AirWindows_Melt,
  AirWindows_MidAmp,
  AirWindows_MidSide,
  AirWindows_Mojo,
  AirWindows_Monitoring,
  AirWindows_Monitoring2,
  AirWindows_Monitoring3,
  AirWindows_MoNoam,
  AirWindows_MSFlipTimer,
  AirWindows_MultiBandDistortion,
  AirWindows_MV,
  AirWindows_MV2,
  AirWindows_NaturalizeDither,
  AirWindows_NCSeventeen,
  AirWindows_Neverland,
  AirWindows_Nikola,
  AirWindows_NodeDither,
  AirWindows_Noise,
  AirWindows_NonlinearSpace,
  AirWindows_NotJustAnotherCD,
  AirWindows_NotJustAnotherDither,
  AirWindows_OneCornerClip,
  AirWindows_OrbitKick,
  AirWindows_Overheads,
  AirWindows_Pafnuty,
  AirWindows_Pafnuty2,
  AirWindows_Parametric,
  AirWindows_PaulDither,
  AirWindows_PaulWide,
  AirWindows_PDBuss,
  AirWindows_PDChannel,
  AirWindows_PeaksOnly,
  AirWindows_Pear,
  AirWindows_Pear2,
  AirWindows_PhaseNudge,
  AirWindows_PitchDelay,
  AirWindows_PitchNasty,
  AirWindows_PlatinumSlew,
  AirWindows_PocketVerbs,
  AirWindows_Pockey,
  AirWindows_Pockey2,
  AirWindows_Podcast,
  AirWindows_PodcastDeluxe,
  AirWindows_Point,
  AirWindows_Pop,
  AirWindows_Pop2,
  AirWindows_Pop3,
  AirWindows_PowerSag,
  AirWindows_PowerSag2,
  AirWindows_Precious,
  AirWindows_Preponderant,
  AirWindows_Pressure4,
  AirWindows_Pressure5,
  AirWindows_PurestAir,
  AirWindows_PurestConsole2Buss,
  AirWindows_PurestConsole2Channel,
  AirWindows_PurestConsole3Buss,
  AirWindows_PurestConsole3Channel,
  AirWindows_PurestConsoleBuss,
  AirWindows_PurestConsoleChannel,
  AirWindows_PurestDrive,
  AirWindows_PurestEcho,
  AirWindows_PurestFade,
  AirWindows_PurestGain,
  AirWindows_PurestSquish,
  AirWindows_PurestWarm,
  AirWindows_PurestWarm2,
  AirWindows_Pyewacket,
  AirWindows_RawGlitters,
  AirWindows_RawTimbers,
  AirWindows_Recurve,
  AirWindows_Remap,
  AirWindows_ResEQ,
  AirWindows_ResEQ2,
  AirWindows_Reverb,
  AirWindows_Righteous4,
  AirWindows_RightoMono,
  AirWindows_SampleDelay,
  AirWindows_Shape,
  AirWindows_ShortBuss,
  AirWindows_SideDull,
  AirWindows_Sidepass,
  AirWindows_Silhouette,
  AirWindows_Sinew,
  AirWindows_SingleEndedTriode,
  AirWindows_Slew,
  AirWindows_Slew2,
  AirWindows_Slew3,
  AirWindows_SlewOnly,
  AirWindows_SlewSonic,
  AirWindows_Smooth,
  AirWindows_SoftGate,
  AirWindows_SpatializeDither,
  AirWindows_Spiral,
  AirWindows_Spiral2,
  AirWindows_Srsly,
  AirWindows_Srsly2,
  AirWindows_Srsly3,
  AirWindows_StarChild,
  AirWindows_StarChild2,
  AirWindows_StereoChorus,
  AirWindows_StereoDoubler,
  AirWindows_StereoEnsemble,
  AirWindows_StereoFX,
  AirWindows_Stonefire,
  AirWindows_StoneFireComp,
  AirWindows_StudioTan,
  AirWindows_SubsOnly,
  AirWindows_SubTight,
  AirWindows_Surge,
  AirWindows_SurgeTide,
  AirWindows_Sweeten,
  AirWindows_Swell,
  AirWindows_Tape,
  AirWindows_TapeDelay,
  AirWindows_TapeDelay2,
  AirWindows_TapeDither,
  AirWindows_TapeDust,
  AirWindows_TapeFat,
  AirWindows_Texturize,
  AirWindows_TexturizeMS,
  AirWindows_Thunder,
  AirWindows_ToneSlant,
  AirWindows_ToTape5,
  AirWindows_ToTape6,
  AirWindows_ToVinyl4,
  AirWindows_TPDFDither,
  AirWindows_TPDFWide,
  AirWindows_TransDesk,
  AirWindows_Tremolo,
  AirWindows_TremoSquare,
  AirWindows_Trianglizer,
  AirWindows_TripleSpread,
  AirWindows_Tube,
  AirWindows_Tube2,
  AirWindows_TubeDesk,
  AirWindows_uLawDecode,
  AirWindows_uLawEncode,
  AirWindows_Ultrasonic,
  AirWindows_UltrasonicLite,
  AirWindows_UltrasonicMed,
  AirWindows_UltrasonX,
  AirWindows_UnBox,
  AirWindows_VariMu,
  AirWindows_Verbity,
  AirWindows_Verbity2,
  AirWindows_Vibrato,
  AirWindows_VinylDither,
  AirWindows_VoiceOfTheStarship,
  AirWindows_VoiceTrick,
  AirWindows_Weight,
  AirWindows_Wider,
  AirWindows_Wolfbot,
  AirWindows_XBandpass,
  AirWindows_XHighpass,
  AirWindows_XLowpass,
  AirWindows_XNotch,
  AirWindows_XRegion,
  AirWindows_YBandpass,
  AirWindows_YHighpass,
  AirWindows_YLowpass,
  AirWindows_YNotBandpass,
  AirWindows_YNotch,
  AirWindows_YNotHighpass,
  AirWindows_YNotLowpass,
  AirWindows_YNotNotch,
  AirWindows_ZBandpass,
  AirWindows_ZBandpass2,
  AirWindows_ZHighpass,
  AirWindows_ZHighpass2,
  AirWindows_ZLowpass,
  AirWindows_ZLowpass2,
  AirWindows_ZNotch,
  AirWindows_ZNotch2,
  AirWindows_ZOutputStage,
  AirWindows_ZRegion,
  AirWindows_ZRegion2,
  AIRWINDOWS_BEGIN = AirWindows_Acceleration,
  AIRWINDOWS_END = AirWindows_ZRegion2,
#endif // SHR3D_SFX_CORE_AIRWINDOWS
#ifdef SHR3D_SFX_CORE_RAKARRACK
  Rakarrack_Reverb,
  Rakarrack_Echo,
  Rakarrack_Chorus,
  Rakarrack_Flanger,
  Rakarrack_Phaser,
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_Overdrive,
  Rakarrack_Distorsion,
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_EQ,
  Rakarrack_ParametricEQ,
  Rakarrack_Cabinet,
  Rakarrack_Compressor,
  Rakarrack_WahWah,
  Rakarrack_AlienWah,
  Rakarrack_Pan,
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
#if 0
  Rakarrack_Harmonizer,
#endif
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_MusicalDelay,
  Rakarrack_NoiseGate,
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_Derelict,
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_AnalogPhaser,
  Rakarrack_Valve,
  Rakarrack_DualFlange,
  Rakarrack_Ring,
  Rakarrack_Exciter,
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_DistBand,
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_Arpie,
  Rakarrack_Expander,
  Rakarrack_Shuffle,
  Rakarrack_Synthfilter,
  Rakarrack_VaryBand,
#ifdef SHR3D_SFX_CORE_RAKARRACK_CONVOLOTRON
  Rakarrack_Convolotron,
#endif // SHR3D_SFX_CORE_RAKARRACK_CONVOLOTRON
#if 0
  Rakarrack_Looper,
#endif
  Rakarrack_MuTroMojo,
  Rakarrack_Echoverse,
  Rakarrack_CoilCrafter,
  Rakarrack_ShelfBoost,
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_Vocoder,
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_Sustainer,
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_Sequence,
  Rakarrack_Shifter,
  Rakarrack_StompBox,
  Rakarrack_Reverbtron,
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_Echotron,
#ifdef SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_StereoHarm,
#endif // SHR3D_SFX_CORE_RAKARRACK_LIBSAMPLERATE
  Rakarrack_CompBand,
  Rakarrack_OpticalTrem,
  Rakarrack_Vibe,
  RAKARRACK_BEGIN = Rakarrack_Reverb,
  RAKARRACK_END = Rakarrack_Vibe,
#endif // SHR3D_SFX_CORE_RAKARRACK
  COUNT
};
#endif // SHR3D_SFX_CORE

struct vec3
{
  union
  {
    f32 x, r = 0.0_f32;
  };
  union
  {
    f32 y, g = 0.0_f32;
  };
  union
  {
    f32 z, b = 0.0_f32;
  };
};

struct vec4
{
  union
  {
    f32 x, r = 0.0_f32;
  };
  union
  {
    f32 y, g = 0.0_f32;
  };
  union
  {
    f32 z, b = 0.0_f32;
  };
  union
  {
    f32 w, a = 0.0_f32;
  };
};

struct mat4
{
  /* 4x4 matrix that looks like this:
   / m00 m01 m02 m03 \
  |  m10 m11 m21 m13  |
  |  m20 m21 m22 m23  |
   \ m30 m31 m32 m33 /
  mat4 is column major, just like what vulkan or opengl expects:*/
  f32 m00 = 1.0_f32, m10 = 0.0_f32, m20 = 0.0_f32, m30 = 0.0_f32;
  f32 m01 = 0.0_f32, m11 = 1.0_f32, m21 = 0.0_f32, m31 = 0.0_f32;
  f32 m02 = 0.0_f32, m12 = 0.0_f32, m22 = 1.0_f32, m32 = 0.0_f32;
  f32 m03 = 0.0_f32, m13 = 0.0_f32, m23 = 0.0_f32, m33 = 1.0_f32;
};

struct BoundingBox
{
  f32 bottom;
  f32 left;
  f32 right;
  f32 top;
};

struct Size
{
  u32 width;
  u32 height;
};

struct Rect
{
  i16 top;
  i16 left;
  i16 bottom;
  i16 right;
};

//struct RectF
//{
//  f32 top;
//  f32 left;
//  f32 bottom;
//  f32 right;
//};

enum struct MidiStatus : u8
{
  chan1NoteOff = 0x80,
  chan1NoteOn = 0x90,
  chan1PitchBendChange = 0xE0
  // there are more.
};

struct MidiMessage
{
  MidiStatus status;
  MidiNote note;
  u8 velocity;
};

struct SongProfile
{
  u64 lastPlayed = 0;
  f32 score = 0.0f;
  i32 timesPlayed = 0;
  i32 toneBank = -1;
  std::vector<std::u8string> toneOverwrites;
};

struct SfxParameterProperties
{
  char name[256];
  char label[64];
  char display[64];
};

struct KeyState
{
  bool pressed : 1 = false; // pressed this frame
  bool released : 1 = false; // released this frame
  bool toggled : 1 = false; // state can be on or off. key press switches state
  bool heldDown : 1 = false; // held down
  bool heldDownLastFrame : 1 = false; // was held down last frame
  bool clickedInUiOrMidi : 1 = false; // the Input::tick() runs before the UI::tick(). This means keypresses update the UI in the same frame, but clicks in the UI are handled one frame later.
};

struct Level
{
  f32 overallLevel;
  std::vector<i32> sectionLevels;
};

#ifdef SHR3D_OPENXR
#define CONTROLLER_EVENT_BIT_value_trigger    0x00000001
#define CONTROLLER_EVENT_BIT_value_squeeze    0x00000002
#define CONTROLLER_EVENT_BIT_value_thumbstick 0x00000004
#define CONTROLLER_EVENT_BIT_click_trigger    0x00000010
#define CONTROLLER_EVENT_BIT_click_thumbstick 0x00000020
#define CONTROLLER_EVENT_BIT_click_squeeze    0x00000040
#define CONTROLLER_EVENT_BIT_click_a          0x00000080
#define CONTROLLER_EVENT_BIT_click_b          0x00000100
#define CONTROLLER_EVENT_BIT_click_x          0x00000200
#define CONTROLLER_EVENT_BIT_click_y          0x00000400
#define CONTROLLER_EVENT_BIT_click_menu       0x00000800
#define CONTROLLER_EVENT_BIT_touch_trigger    0x00001000
#define CONTROLLER_EVENT_BIT_touch_thumbstick 0x00002000
#define CONTROLLER_EVENT_BIT_touch_a          0x00004000
#define CONTROLLER_EVENT_BIT_touch_b          0x00008000
#define CONTROLLER_EVENT_BIT_touch_x          0x00010000
#define CONTROLLER_EVENT_BIT_touch_y          0x00020000
struct XrControllerEvent
{
  u32 controllerEventBit;
  f32 trigger;         // 0.0f - 1.0f
  f32 squeeze;         // 0.0f - 1.0f
  f32 thumbstick_x;    //-1.0f - 1.0f
  f32 thumbstick_y;    //-1.0f - 1.0f
  bool click_trigger;    //true:donw false:up
  bool click_thumbstick; //true:donw false:up  Surprise, the thumbstick can be pressed
  bool click_squeeze;    //true:donw false:up
  bool click_a;          //true:donw false:up
  bool click_b;          //true:donw false:up
  bool click_x;          //true:donw false:up
  bool click_y;          //true:donw false:up
  bool click_menu;       //true:donw false:up
  // seldom use
  bool touch_trigger;    //true:touch false:none touch
  bool touch_thumbstick; //true:touch false:none touch
  bool touch_a;          //true:touch false:none touch
  bool touch_b;          //true:touch false:none touch
  bool touch_x;          //true:touch false:none touch
  bool touch_y;          //true:touch false:none touch
};

enum struct XrActiveController : u8 {
  left,
  right,
  mouseKeyboard
};

#ifdef SHR3D_OPENXR_CONTROLLER_PICO4_AND_QUEST3
enum struct XrController : u8 {
  pico4,
  quest3
};
#endif // SHR3D_OPENXR_CONTROLLER_PICO4_AND_QUEST3
#endif // SHR3D_OPENXR


#ifdef SHR3D_ENVIRONMENT_SKYBOX_CUBE_TEXTURE
enum struct SkyboxTextureType : u8 {
  defaultRaw,
  cubemapRaw,
  defaultDds,
  cubemapDds
};
#endif // SHR3D_ENVIRONMENT_SKYBOX_CUBE_TEXTURE

#ifdef SHR3D_PARTICLE
struct Particle_
{
  TimeNS particleBornTime;
  vec3 bornPosition;
  vec3 velocity;
  vec3 acceleration;
  f32 rotation;
  vec4 color;
  f32 scale;
};
#endif // SHR3D_PARTICLE

#ifdef SHR3D_SFX_PLUGIN_BENCHMARK
struct SfxBenchmarkResult
{
  TimeNS duration;
  u64 memoryUsageInBytes;
};
#endif // SHR3D_SFX_PLUGIN_BENCHMARK

#endif // TYPE_H
